/*
 * Copyright (C) 2019 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/encoding/range-byte.py"!
 * Do not change this file!
 */


#ifndef LEXBOR_ENCODING_RANGE_H
#define LEXBOR_ENCODING_RANGE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "lexbor/encoding/base.h"


#define LXB_ENCODING_RANGE_INDEX_GB18030_SIZE 207


LXB_EXTERN const lxb_encoding_range_index_t lxb_encoding_range_index_gb18030[207];


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* LEXBOR_ENCODING_RANGE_H */
