<?php

/** @generate-class-entries */

namespace Pdo;

/**
 * @strict-properties
 * @not-serializable
 */
class Dblib extends \PDO
{
    /** @cvalue PDO_DBLIB_ATTR_CONNECTION_TIMEOUT */
    public const int ATTR_CONNECTION_TIMEOUT = UNKNOWN;

    /** @cvalue PDO_DBLIB_ATTR_QUERY_TIMEOUT */
    public const int ATTR_QUERY_TIMEOUT = UNKNOWN;

    /** @cvalue PDO_DBLIB_ATTR_STRINGIFY_UNIQUEIDENTIFIER */
    public const int ATTR_STRINGIFY_UNIQUEIDENTIFIER = UNKNOWN;

    /** @cvalue PDO_DBLIB_ATTR_VERSION */
    public const int ATTR_VERSION = UNKNOWN;

    /** @cvalue PDO_DBLIB_ATTR_TDS_VERSION */
    public const int ATTR_TDS_VERSION = UNKNOWN;

    /** @cvalue PDO_DBLIB_ATTR_SKIP_EMPTY_ROWSETS */
    public const int ATTR_SKIP_EMPTY_ROWSETS = UNKNOWN;

    /** @cvalue PDO_DBLIB_ATTR_DATETIME_CONVERT */
    public const int ATTR_DATETIME_CONVERT = UNKNOWN;
}
