/*
 * Copyright (C) 2018 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

/*
 * Caution!
 * This file generated by the script "utils/lexbor/tag_ns/tags.py"!
 * Do not change this file!
 */


#ifndef LXB_NS_CONST_H
#define LXB_NS_CONST_H

#define LXB_NS_CONST_VERSION "253D4AFDA959234B48A478B956C3C777"
#define LXB_NS_CONST_VERSION_253D4AFDA959234B48A478B956C3C777

typedef uintptr_t lxb_ns_id_t;
typedef uintptr_t lxb_ns_prefix_id_t;

typedef enum {
    LXB_NS__UNDEF      = 0x00,
    LXB_NS__ANY        = 0x01,
    LXB_NS_HTML        = 0x02,
    LXB_NS_MATH        = 0x03,
    LXB_NS_SVG         = 0x04,
    LXB_NS_XLINK       = 0x05,
    LXB_NS_XML         = 0x06,
    LXB_NS_XMLNS       = 0x07,
    LXB_NS__LAST_ENTRY = 0x08
}
lxb_ns_id_enum_t;


#endif /* LXB_NS_CONST_H */
