<?php

/** @generate-class-entries */

/* Warnings */

/**
 * @var int
 * @cvalue U_USING_FALLBACK_WARNING
 */
const U_USING_FALLBACK_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_ERROR_WARNING_START
 */
const U_ERROR_WARNING_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_USING_DEFAULT_WARNING
 */
const U_USING_DEFAULT_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_SAFECLONE_ALLOCATED_WARNING
 */
const U_SAFECLONE_ALLOCATED_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_STATE_OLD_WARNING
 */
const U_STATE_OLD_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_STRING_NOT_TERMINATED_WARNING
 */
const U_STRING_NOT_TERMINATED_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_SORT_KEY_TOO_SHORT_WARNING
 */
const U_SORT_KEY_TOO_SHORT_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_AMBIGUOUS_ALIAS_WARNING
 */
const U_AMBIGUOUS_ALIAS_WARNING = UNKNOWN;
/**
 * @var int
 * @cvalue U_DIFFERENT_UCA_VERSION
 */
const U_DIFFERENT_UCA_VERSION = UNKNOWN;
/**
 * @var int
 * @cvalue U_ERROR_WARNING_LIMIT
 */
const U_ERROR_WARNING_LIMIT = UNKNOWN;

/* Standard errors */
/**
 * @var int
 * @cvalue U_ZERO_ERROR
 */
const U_ZERO_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_ILLEGAL_ARGUMENT_ERROR
 */
const U_ILLEGAL_ARGUMENT_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISSING_RESOURCE_ERROR
 */
const U_MISSING_RESOURCE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_FORMAT_ERROR
 */
const U_INVALID_FORMAT_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_FILE_ACCESS_ERROR
 */
const U_FILE_ACCESS_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INTERNAL_PROGRAM_ERROR
 */
const U_INTERNAL_PROGRAM_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_MESSAGE_PARSE_ERROR
 */
const U_MESSAGE_PARSE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_MEMORY_ALLOCATION_ERROR
 */
const U_MEMORY_ALLOCATION_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INDEX_OUTOFBOUNDS_ERROR
 */
const U_INDEX_OUTOFBOUNDS_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_PARSE_ERROR
 */
const U_PARSE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_CHAR_FOUND
 */
const U_INVALID_CHAR_FOUND = UNKNOWN;
/**
 * @var int
 * @cvalue U_TRUNCATED_CHAR_FOUND
 */
const U_TRUNCATED_CHAR_FOUND = UNKNOWN;
/**
 * @var int
 * @cvalue U_ILLEGAL_CHAR_FOUND
 */
const U_ILLEGAL_CHAR_FOUND = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_TABLE_FORMAT
 */
const U_INVALID_TABLE_FORMAT = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_TABLE_FILE
 */
const U_INVALID_TABLE_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue U_BUFFER_OVERFLOW_ERROR
 */
const U_BUFFER_OVERFLOW_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNSUPPORTED_ERROR
 */
const U_UNSUPPORTED_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_RESOURCE_TYPE_MISMATCH
 */
const U_RESOURCE_TYPE_MISMATCH = UNKNOWN;
/**
 * @var int
 * @cvalue U_ILLEGAL_ESCAPE_SEQUENCE
 */
const U_ILLEGAL_ESCAPE_SEQUENCE = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNSUPPORTED_ESCAPE_SEQUENCE
 */
const U_UNSUPPORTED_ESCAPE_SEQUENCE = UNKNOWN;
/**
 * @var int
 * @cvalue U_NO_SPACE_AVAILABLE
 */
const U_NO_SPACE_AVAILABLE = UNKNOWN;
/**
 * @var int
 * @cvalue U_CE_NOT_FOUND_ERROR
 */
const U_CE_NOT_FOUND_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_PRIMARY_TOO_LONG_ERROR
 */
const U_PRIMARY_TOO_LONG_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_STATE_TOO_OLD_ERROR
 */
const U_STATE_TOO_OLD_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_TOO_MANY_ALIASES_ERROR
 */
const U_TOO_MANY_ALIASES_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_ENUM_OUT_OF_SYNC_ERROR
 */
const U_ENUM_OUT_OF_SYNC_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVARIANT_CONVERSION_ERROR
 */
const U_INVARIANT_CONVERSION_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_STATE_ERROR
 */
const U_INVALID_STATE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_COLLATOR_VERSION_MISMATCH
 */
const U_COLLATOR_VERSION_MISMATCH = UNKNOWN;
/**
 * @var int
 * @cvalue U_USELESS_COLLATOR_ERROR
 */
const U_USELESS_COLLATOR_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_NO_WRITE_PERMISSION
 */
const U_NO_WRITE_PERMISSION = UNKNOWN;
/**
 * @var int
 * @cvalue U_STANDARD_ERROR_LIMIT
 */
const U_STANDARD_ERROR_LIMIT = UNKNOWN;

/* The error code range 0x10000 0x10100 are reserved for Transliterator */

/**
 * @var int
 * @cvalue U_BAD_VARIABLE_DEFINITION
 */
const U_BAD_VARIABLE_DEFINITION = UNKNOWN;
/**
 * @var int
 * @cvalue U_PARSE_ERROR_START
 */
const U_PARSE_ERROR_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_RULE
 */
const U_MALFORMED_RULE = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_SET
 */
const U_MALFORMED_SET = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_SYMBOL_REFERENCE
 */
const U_MALFORMED_SYMBOL_REFERENCE = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_UNICODE_ESCAPE
 */
const U_MALFORMED_UNICODE_ESCAPE = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_VARIABLE_DEFINITION
 */
const U_MALFORMED_VARIABLE_DEFINITION = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_VARIABLE_REFERENCE
 */
const U_MALFORMED_VARIABLE_REFERENCE = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISMATCHED_SEGMENT_DELIMITERS
 */
const U_MISMATCHED_SEGMENT_DELIMITERS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISPLACED_ANCHOR_START
 */
const U_MISPLACED_ANCHOR_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISPLACED_CURSOR_OFFSET
 */
const U_MISPLACED_CURSOR_OFFSET = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISPLACED_QUANTIFIER
 */
const U_MISPLACED_QUANTIFIER = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISSING_OPERATOR
 */
const U_MISSING_OPERATOR = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISSING_SEGMENT_CLOSE
 */
const U_MISSING_SEGMENT_CLOSE = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_ANTE_CONTEXTS
 */
const U_MULTIPLE_ANTE_CONTEXTS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_CURSORS
 */
const U_MULTIPLE_CURSORS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_POST_CONTEXTS
 */
const U_MULTIPLE_POST_CONTEXTS = UNKNOWN;
/**
 * @var int
 * @cvalue U_TRAILING_BACKSLASH
 */
const U_TRAILING_BACKSLASH = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNDEFINED_SEGMENT_REFERENCE
 */
const U_UNDEFINED_SEGMENT_REFERENCE = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNDEFINED_VARIABLE
 */
const U_UNDEFINED_VARIABLE = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNQUOTED_SPECIAL
 */
const U_UNQUOTED_SPECIAL = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNTERMINATED_QUOTE
 */
const U_UNTERMINATED_QUOTE = UNKNOWN;
/**
 * @var int
 * @cvalue U_RULE_MASK_ERROR
 */
const U_RULE_MASK_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_MISPLACED_COMPOUND_FILTER
 */
const U_MISPLACED_COMPOUND_FILTER = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_COMPOUND_FILTERS
 */
const U_MULTIPLE_COMPOUND_FILTERS = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_RBT_SYNTAX
 */
const U_INVALID_RBT_SYNTAX = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_PROPERTY_PATTERN
 */
const U_INVALID_PROPERTY_PATTERN = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_PRAGMA
 */
const U_MALFORMED_PRAGMA = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNCLOSED_SEGMENT
 */
const U_UNCLOSED_SEGMENT = UNKNOWN;
/**
 * @var int
 * @cvalue U_ILLEGAL_CHAR_IN_SEGMENT
 */
const U_ILLEGAL_CHAR_IN_SEGMENT = UNKNOWN;
/**
 * @var int
 * @cvalue U_VARIABLE_RANGE_EXHAUSTED
 */
const U_VARIABLE_RANGE_EXHAUSTED = UNKNOWN;
/**
 * @var int
 * @cvalue U_VARIABLE_RANGE_OVERLAP
 */
const U_VARIABLE_RANGE_OVERLAP = UNKNOWN;
/**
 * @var int
 * @cvalue U_ILLEGAL_CHARACTER
 */
const U_ILLEGAL_CHARACTER = UNKNOWN;
/**
 * @var int
 * @cvalue U_INTERNAL_TRANSLITERATOR_ERROR
 */
const U_INTERNAL_TRANSLITERATOR_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_ID
 */
const U_INVALID_ID = UNKNOWN;
/**
 * @var int
 * @cvalue U_INVALID_FUNCTION
 */
const U_INVALID_FUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue U_PARSE_ERROR_LIMIT
 */
const U_PARSE_ERROR_LIMIT = UNKNOWN;

/* The error code range 0x10100 0x10200 are reserved for formatting API parsing errors */

/**
 * @var int
 * @cvalue U_UNEXPECTED_TOKEN
 */
const U_UNEXPECTED_TOKEN = UNKNOWN;
/**
 * @var int
 * @cvalue U_FMT_PARSE_ERROR_START
 */
const U_FMT_PARSE_ERROR_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_DECIMAL_SEPARATORS
 */
const U_MULTIPLE_DECIMAL_SEPARATORS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_DECIMAL_SEPERATORS
 */
#[\Deprecated(since: '8.3', message: 'use U_MULTIPLE_DECIMAL_SEPARATORS instead')]
const U_MULTIPLE_DECIMAL_SEPERATORS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_EXPONENTIAL_SYMBOLS
 */
const U_MULTIPLE_EXPONENTIAL_SYMBOLS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MALFORMED_EXPONENTIAL_PATTERN
 */
const U_MALFORMED_EXPONENTIAL_PATTERN = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_PERCENT_SYMBOLS
 */
const U_MULTIPLE_PERCENT_SYMBOLS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_PERMILL_SYMBOLS
 */
const U_MULTIPLE_PERMILL_SYMBOLS = UNKNOWN;
/**
 * @var int
 * @cvalue U_MULTIPLE_PAD_SPECIFIERS
 */
const U_MULTIPLE_PAD_SPECIFIERS = UNKNOWN;
/**
 * @var int
 * @cvalue U_PATTERN_SYNTAX_ERROR
 */
const U_PATTERN_SYNTAX_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_ILLEGAL_PAD_POSITION
 */
const U_ILLEGAL_PAD_POSITION = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNMATCHED_BRACES
 */
const U_UNMATCHED_BRACES = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNSUPPORTED_PROPERTY
 */
const U_UNSUPPORTED_PROPERTY = UNKNOWN;
/**
 * @var int
 * @cvalue U_UNSUPPORTED_ATTRIBUTE
 */
const U_UNSUPPORTED_ATTRIBUTE = UNKNOWN;
/**
 * @var int
 * @cvalue U_FMT_PARSE_ERROR_LIMIT
 */
const U_FMT_PARSE_ERROR_LIMIT = UNKNOWN;

/* The error code range 0x10200 0x102ff are reserved for Break Iterator related errors */

/**
 * @var int
 * @cvalue U_BRK_INTERNAL_ERROR
 */
const U_BRK_INTERNAL_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_ERROR_START
 */
const U_BRK_ERROR_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_HEX_DIGITS_EXPECTED
 */
const U_BRK_HEX_DIGITS_EXPECTED = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_SEMICOLON_EXPECTED
 */
const U_BRK_SEMICOLON_EXPECTED = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_RULE_SYNTAX
 */
const U_BRK_RULE_SYNTAX = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_UNCLOSED_SET
 */
const U_BRK_UNCLOSED_SET = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_ASSIGN_ERROR
 */
const U_BRK_ASSIGN_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_VARIABLE_REDFINITION
 */
const U_BRK_VARIABLE_REDFINITION = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_MISMATCHED_PAREN
 */
const U_BRK_MISMATCHED_PAREN = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_NEW_LINE_IN_QUOTED_STRING
 */
const U_BRK_NEW_LINE_IN_QUOTED_STRING = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_UNDEFINED_VARIABLE
 */
const U_BRK_UNDEFINED_VARIABLE = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_INIT_ERROR
 */
const U_BRK_INIT_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_RULE_EMPTY_SET
 */
const U_BRK_RULE_EMPTY_SET = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_UNRECOGNIZED_OPTION
 */
const U_BRK_UNRECOGNIZED_OPTION = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_MALFORMED_RULE_TAG
 */
const U_BRK_MALFORMED_RULE_TAG = UNKNOWN;
/**
 * @var int
 * @cvalue U_BRK_ERROR_LIMIT
 */
const U_BRK_ERROR_LIMIT = UNKNOWN;

/* The error codes in the range 0x10300-0x103ff are reserved for regular expression related errors */

/**
 * @var int
 * @cvalue U_REGEX_INTERNAL_ERROR
 */
const U_REGEX_INTERNAL_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_ERROR_START
 */
const U_REGEX_ERROR_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_RULE_SYNTAX
 */
const U_REGEX_RULE_SYNTAX = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_INVALID_STATE
 */
const U_REGEX_INVALID_STATE = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_BAD_ESCAPE_SEQUENCE
 */
const U_REGEX_BAD_ESCAPE_SEQUENCE = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_PROPERTY_SYNTAX
 */
const U_REGEX_PROPERTY_SYNTAX = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_UNIMPLEMENTED
 */
const U_REGEX_UNIMPLEMENTED = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_MISMATCHED_PAREN
 */
const U_REGEX_MISMATCHED_PAREN = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_NUMBER_TOO_BIG
 */
const U_REGEX_NUMBER_TOO_BIG = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_BAD_INTERVAL
 */
const U_REGEX_BAD_INTERVAL = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_MAX_LT_MIN
 */
const U_REGEX_MAX_LT_MIN = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_INVALID_BACK_REF
 */
const U_REGEX_INVALID_BACK_REF = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_INVALID_FLAG
 */
const U_REGEX_INVALID_FLAG = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_LOOK_BEHIND_LIMIT
 */
const U_REGEX_LOOK_BEHIND_LIMIT = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_SET_CONTAINS_STRING
 */
const U_REGEX_SET_CONTAINS_STRING = UNKNOWN;
/**
 * @var int
 * @cvalue U_REGEX_ERROR_LIMIT
 */
const U_REGEX_ERROR_LIMIT = UNKNOWN;

/* The error code in the range 0x10400-0x104ff are reserved for IDNA related error codes */

/**
 * @var int
 * @cvalue U_IDNA_PROHIBITED_ERROR
 */
const U_IDNA_PROHIBITED_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_ERROR_START
 */
const U_IDNA_ERROR_START = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_UNASSIGNED_ERROR
 */
const U_IDNA_UNASSIGNED_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_CHECK_BIDI_ERROR
 */
const U_IDNA_CHECK_BIDI_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_STD3_ASCII_RULES_ERROR
 */
const U_IDNA_STD3_ASCII_RULES_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_ACE_PREFIX_ERROR
 */
const U_IDNA_ACE_PREFIX_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_VERIFICATION_ERROR
 */
const U_IDNA_VERIFICATION_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_LABEL_TOO_LONG_ERROR
 */
const U_IDNA_LABEL_TOO_LONG_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_ZERO_LENGTH_LABEL_ERROR
 */
const U_IDNA_ZERO_LENGTH_LABEL_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR
 */
const U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_IDNA_ERROR_LIMIT
 */
const U_IDNA_ERROR_LIMIT = UNKNOWN;

/* Aliases for StringPrep */

/**
 * @var int
 * @cvalue U_STRINGPREP_PROHIBITED_ERROR
 */
const U_STRINGPREP_PROHIBITED_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_STRINGPREP_UNASSIGNED_ERROR
 */
const U_STRINGPREP_UNASSIGNED_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue U_STRINGPREP_CHECK_BIDI_ERROR
 */
const U_STRINGPREP_CHECK_BIDI_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue U_ERROR_LIMIT
 */
const U_ERROR_LIMIT = UNKNOWN;

/** @not-serializable */
class IntlIterator implements Iterator
{
    /** @tentative-return-type */
    public function current(): mixed {}

    /** @tentative-return-type */
    public function key(): mixed {}

    /** @tentative-return-type */
    public function next(): void {}

    /** @tentative-return-type */
    public function rewind(): void {}

    /** @tentative-return-type */
    public function valid(): bool {}
}
