#include <stddef.h>
#include "nls_hy.h"

const mbfl_language mbfl_language_armenian = {
	mbfl_no_language_armenian ,
	"Armenian",
	"hy",
	NULL,
	mbfl_no_encoding_armscii8,
	mbfl_no_encoding_qprint,
	mbfl_no_encoding_8bit
};
