<?php

/** @generate-class-entries */

/* See http://curl.haxx.se/lxr/source/docs/libcurl/symbols-in-versions
   or curl src/docs/libcurl/symbols-in-versions for a (almost) complete list
   of options and which version they were introduced */

/**
 * @var int
 * @cvalue CURLOPT_AUTOREFERER
 */
const CURLOPT_AUTOREFERER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_BINARYTRANSFER
 */
#[\Deprecated(since: '8.4', message: 'as it had no effect since 5.1.2')]
const CURLOPT_BINARYTRANSFER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_BUFFERSIZE
 */
const CURLOPT_BUFFERSIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CAINFO
 */
const CURLOPT_CAINFO = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CAPATH
 */
const CURLOPT_CAPATH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CONNECTTIMEOUT
 */
const CURLOPT_CONNECTTIMEOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_COOKIE
 */
const CURLOPT_COOKIE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_COOKIEFILE
 */
const CURLOPT_COOKIEFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_COOKIEJAR
 */
const CURLOPT_COOKIEJAR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_COOKIESESSION
 */
const CURLOPT_COOKIESESSION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CRLF
 */
const CURLOPT_CRLF = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CUSTOMREQUEST
 */
const CURLOPT_CUSTOMREQUEST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DNS_CACHE_TIMEOUT
 */
const CURLOPT_DNS_CACHE_TIMEOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DNS_USE_GLOBAL_CACHE
 */
const CURLOPT_DNS_USE_GLOBAL_CACHE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_EGDSOCKET
 */
const CURLOPT_EGDSOCKET = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_ACCEPT_ENCODING
 * @alias CURLOPT_ACCEPT_ENCODING
 */
const CURLOPT_ENCODING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FAILONERROR
 */
const CURLOPT_FAILONERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FILE
 */
const CURLOPT_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FILETIME
 */
const CURLOPT_FILETIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FOLLOWLOCATION
 */
const CURLOPT_FOLLOWLOCATION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FORBID_REUSE
 */
const CURLOPT_FORBID_REUSE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FRESH_CONNECT
 */
const CURLOPT_FRESH_CONNECT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTPAPPEND
 */
const CURLOPT_FTPAPPEND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTPLISTONLY
 */
const CURLOPT_FTPLISTONLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTPPORT
 */
const CURLOPT_FTPPORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTP_USE_EPRT
 */
const CURLOPT_FTP_USE_EPRT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTP_USE_EPSV
 */
const CURLOPT_FTP_USE_EPSV = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HEADER
 */
const CURLOPT_HEADER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HEADERFUNCTION
 */
const CURLOPT_HEADERFUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTP200ALIASES
 */
const CURLOPT_HTTP200ALIASES = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTPGET
 */
const CURLOPT_HTTPGET = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTPHEADER
 */
const CURLOPT_HTTPHEADER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTPPROXYTUNNEL
 */
const CURLOPT_HTTPPROXYTUNNEL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTP_VERSION
 */
const CURLOPT_HTTP_VERSION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_INFILE
 */
const CURLOPT_INFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_INFILESIZE
 */
const CURLOPT_INFILESIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_INFILESIZE_LARGE
 */
const CURLOPT_INFILESIZE_LARGE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_INTERFACE
 */
const CURLOPT_INTERFACE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_KRB4LEVEL
 */
const CURLOPT_KRB4LEVEL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_LOW_SPEED_LIMIT
 */
const CURLOPT_LOW_SPEED_LIMIT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_LOW_SPEED_TIME
 */
const CURLOPT_LOW_SPEED_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAXCONNECTS
 */
const CURLOPT_MAXCONNECTS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAXREDIRS
 */
const CURLOPT_MAXREDIRS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NETRC
 */
const CURLOPT_NETRC = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NOBODY
 */
const CURLOPT_NOBODY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NOPROGRESS
 */
const CURLOPT_NOPROGRESS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NOSIGNAL
 */
const CURLOPT_NOSIGNAL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PORT
 */
const CURLOPT_PORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_POST
 */
const CURLOPT_POST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_POSTFIELDS
 */
const CURLOPT_POSTFIELDS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_POSTQUOTE
 */
const CURLOPT_POSTQUOTE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PREQUOTE
 */
const CURLOPT_PREQUOTE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PRIVATE
 */
const CURLOPT_PRIVATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROGRESSFUNCTION
 */
const CURLOPT_PROGRESSFUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY
 */
const CURLOPT_PROXY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYPORT
 */
const CURLOPT_PROXYPORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYTYPE
 */
const CURLOPT_PROXYTYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYUSERPWD
 */
const CURLOPT_PROXYUSERPWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PUT
 */
const CURLOPT_PUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_QUOTE
 */
const CURLOPT_QUOTE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RANDOM_FILE
 */
const CURLOPT_RANDOM_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RANGE
 */
const CURLOPT_RANGE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_READDATA
 */
const CURLOPT_READDATA = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_READFUNCTION
 */
const CURLOPT_READFUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_REFERER
 */
const CURLOPT_REFERER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RESUME_FROM
 */
const CURLOPT_RESUME_FROM = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RETURNTRANSFER
 */
const CURLOPT_RETURNTRANSFER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SHARE
 */
const CURLOPT_SHARE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLCERT
 */
const CURLOPT_SSLCERT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLCERTPASSWD
 */
const CURLOPT_SSLCERTPASSWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLCERTTYPE
 */
const CURLOPT_SSLCERTTYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLENGINE
 */
const CURLOPT_SSLENGINE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLENGINE_DEFAULT
 */
const CURLOPT_SSLENGINE_DEFAULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLKEY
 */
const CURLOPT_SSLKEY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLKEYPASSWD
 */
const CURLOPT_SSLKEYPASSWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLKEYTYPE
 */
const CURLOPT_SSLKEYTYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLVERSION
 */
const CURLOPT_SSLVERSION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_CIPHER_LIST
 */
const CURLOPT_SSL_CIPHER_LIST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_VERIFYHOST
 */
const CURLOPT_SSL_VERIFYHOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_VERIFYPEER
 */
const CURLOPT_SSL_VERIFYPEER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_STDERR
 */
const CURLOPT_STDERR = UNKNOWN;
#if LIBCURL_VERSION_NUM >= 0x080900 /* Available since 8.9.0 */
/**
 * @var int
 * @cvalue CURLOPT_TCP_KEEPCNT
 */
const CURLOPT_TCP_KEEPCNT = UNKNOWN;
#endif
/**
 * @var int
 * @cvalue CURLOPT_TELNETOPTIONS
 */
const CURLOPT_TELNETOPTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TIMECONDITION
 */
const CURLOPT_TIMECONDITION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TIMEOUT
 */
const CURLOPT_TIMEOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TIMEVALUE
 */
const CURLOPT_TIMEVALUE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TRANSFERTEXT
 */
const CURLOPT_TRANSFERTEXT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_UNRESTRICTED_AUTH
 */
const CURLOPT_UNRESTRICTED_AUTH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_UPLOAD
 */
const CURLOPT_UPLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_URL
 */
const CURLOPT_URL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_USERAGENT
 */
const CURLOPT_USERAGENT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_USERPWD
 */
const CURLOPT_USERPWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_VERBOSE
 */
const CURLOPT_VERBOSE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_WRITEFUNCTION
 */
const CURLOPT_WRITEFUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_WRITEHEADER
 */
const CURLOPT_WRITEHEADER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_XFERINFOFUNCTION
 */
const CURLOPT_XFERINFOFUNCTION = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_DEBUGFUNCTION
 */
const CURLOPT_DEBUGFUNCTION = UNKNOWN;

#if LIBCURL_VERSION_NUM >= 0x080d00 /* Available since 8.13.0 */
/**
 * @var int
 * @cvalue CURLFOLLOW_ALL
 */
const CURLFOLLOW_ALL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFOLLOW_OBEYCODE
 */
const CURLFOLLOW_OBEYCODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFOLLOW_FIRSTONLY
 */
const CURLFOLLOW_FIRSTONLY = UNKNOWN;
#endif

/**
 * @var int
 * @cvalue CURLINFO_TEXT
 */
const CURLINFO_TEXT = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_HEADER_IN
 */
const CURLINFO_HEADER_IN = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_DATA_IN
 */
const CURLINFO_DATA_IN = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_DATA_OUT
 */
const CURLINFO_DATA_OUT = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_SSL_DATA_OUT
 */
const CURLINFO_SSL_DATA_OUT = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_SSL_DATA_IN
 */
const CURLINFO_SSL_DATA_IN = UNKNOWN;

/* */
/**
 * @var int
 * @cvalue CURLE_ABORTED_BY_CALLBACK
 */
const CURLE_ABORTED_BY_CALLBACK = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_BAD_CALLING_ORDER
 */
const CURLE_BAD_CALLING_ORDER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_BAD_CONTENT_ENCODING
 */
const CURLE_BAD_CONTENT_ENCODING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_BAD_DOWNLOAD_RESUME
 */
const CURLE_BAD_DOWNLOAD_RESUME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_BAD_FUNCTION_ARGUMENT
 */
const CURLE_BAD_FUNCTION_ARGUMENT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_BAD_PASSWORD_ENTERED
 */
const CURLE_BAD_PASSWORD_ENTERED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_COULDNT_CONNECT
 */
const CURLE_COULDNT_CONNECT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_COULDNT_RESOLVE_HOST
 */
const CURLE_COULDNT_RESOLVE_HOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_COULDNT_RESOLVE_PROXY
 */
const CURLE_COULDNT_RESOLVE_PROXY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FAILED_INIT
 */
const CURLE_FAILED_INIT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FILE_COULDNT_READ_FILE
 */
const CURLE_FILE_COULDNT_READ_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_ACCESS_DENIED
 */
const CURLE_FTP_ACCESS_DENIED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_BAD_DOWNLOAD_RESUME
 */
const CURLE_FTP_BAD_DOWNLOAD_RESUME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_CANT_GET_HOST
 */
const CURLE_FTP_CANT_GET_HOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_CANT_RECONNECT
 */
const CURLE_FTP_CANT_RECONNECT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_COULDNT_GET_SIZE
 */
const CURLE_FTP_COULDNT_GET_SIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_COULDNT_RETR_FILE
 */
const CURLE_FTP_COULDNT_RETR_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_COULDNT_SET_ASCII
 */
const CURLE_FTP_COULDNT_SET_ASCII = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_COULDNT_SET_BINARY
 */
const CURLE_FTP_COULDNT_SET_BINARY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_COULDNT_STOR_FILE
 */
const CURLE_FTP_COULDNT_STOR_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_COULDNT_USE_REST
 */
const CURLE_FTP_COULDNT_USE_REST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_PARTIAL_FILE
 */
const CURLE_FTP_PARTIAL_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_PORT_FAILED
 */
const CURLE_FTP_PORT_FAILED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_QUOTE_ERROR
 */
const CURLE_FTP_QUOTE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_USER_PASSWORD_INCORRECT
 */
const CURLE_FTP_USER_PASSWORD_INCORRECT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_WEIRD_227_FORMAT
 */
const CURLE_FTP_WEIRD_227_FORMAT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_WEIRD_PASS_REPLY
 */
const CURLE_FTP_WEIRD_PASS_REPLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_WEIRD_PASV_REPLY
 */
const CURLE_FTP_WEIRD_PASV_REPLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_WEIRD_SERVER_REPLY
 */
const CURLE_FTP_WEIRD_SERVER_REPLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_WEIRD_USER_REPLY
 */
const CURLE_FTP_WEIRD_USER_REPLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FTP_WRITE_ERROR
 */
const CURLE_FTP_WRITE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_FUNCTION_NOT_FOUND
 */
const CURLE_FUNCTION_NOT_FOUND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_GOT_NOTHING
 */
const CURLE_GOT_NOTHING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_HTTP_NOT_FOUND
 */
const CURLE_HTTP_NOT_FOUND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_HTTP_PORT_FAILED
 */
const CURLE_HTTP_PORT_FAILED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_HTTP_POST_ERROR
 */
const CURLE_HTTP_POST_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_HTTP_RANGE_ERROR
 */
const CURLE_HTTP_RANGE_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_HTTP_RETURNED_ERROR
 */
const CURLE_HTTP_RETURNED_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_LDAP_CANNOT_BIND
 */
const CURLE_LDAP_CANNOT_BIND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_LDAP_SEARCH_FAILED
 */
const CURLE_LDAP_SEARCH_FAILED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_LIBRARY_NOT_FOUND
 */
const CURLE_LIBRARY_NOT_FOUND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_MALFORMAT_USER
 */
const CURLE_MALFORMAT_USER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_OBSOLETE
 */
const CURLE_OBSOLETE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_OK
 */
const CURLE_OK = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_OPERATION_TIMEDOUT
 */
const CURLE_OPERATION_TIMEDOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_OPERATION_TIMEOUTED
 */
const CURLE_OPERATION_TIMEOUTED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_OUT_OF_MEMORY
 */
const CURLE_OUT_OF_MEMORY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_PARTIAL_FILE
 */
const CURLE_PARTIAL_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_READ_ERROR
 */
const CURLE_READ_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_RECV_ERROR
 */
const CURLE_RECV_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SEND_ERROR
 */
const CURLE_SEND_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SHARE_IN_USE
 */
const CURLE_SHARE_IN_USE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_CACERT
 */
const CURLE_SSL_CACERT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_CERTPROBLEM
 */
const CURLE_SSL_CERTPROBLEM = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_CIPHER
 */
const CURLE_SSL_CIPHER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_CONNECT_ERROR
 */
const CURLE_SSL_CONNECT_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_ENGINE_NOTFOUND
 */
const CURLE_SSL_ENGINE_NOTFOUND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_ENGINE_SETFAILED
 */
const CURLE_SSL_ENGINE_SETFAILED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_PEER_CERTIFICATE
 */
const CURLE_SSL_PEER_CERTIFICATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_SSL_PINNEDPUBKEYNOTMATCH
 */
const CURLE_SSL_PINNEDPUBKEYNOTMATCH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_TELNET_OPTION_SYNTAX
 */
const CURLE_TELNET_OPTION_SYNTAX = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_TOO_MANY_REDIRECTS
 */
const CURLE_TOO_MANY_REDIRECTS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_UNKNOWN_TELNET_OPTION
 */
const CURLE_UNKNOWN_TELNET_OPTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_UNSUPPORTED_PROTOCOL
 */
const CURLE_UNSUPPORTED_PROTOCOL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_URL_MALFORMAT
 */
const CURLE_URL_MALFORMAT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_URL_MALFORMAT_USER
 */
const CURLE_URL_MALFORMAT_USER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_WRITE_ERROR
 */
const CURLE_WRITE_ERROR = UNKNOWN;

/* cURL info constants */
/**
 * @var int
 * @cvalue CURLINFO_CONNECT_TIME
 */
const CURLINFO_CONNECT_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_CONTENT_LENGTH_DOWNLOAD
 */
const CURLINFO_CONTENT_LENGTH_DOWNLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_CONTENT_LENGTH_UPLOAD
 */
const CURLINFO_CONTENT_LENGTH_UPLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_CONTENT_TYPE
 */
const CURLINFO_CONTENT_TYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_EFFECTIVE_URL
 */
const CURLINFO_EFFECTIVE_URL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_FILETIME
 */
const CURLINFO_FILETIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_HEADER_OUT
 */
const CURLINFO_HEADER_OUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_HEADER_SIZE
 */
const CURLINFO_HEADER_SIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_HTTP_CODE
 */
const CURLINFO_HTTP_CODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_LASTONE
 */
const CURLINFO_LASTONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_NAMELOOKUP_TIME
 */
const CURLINFO_NAMELOOKUP_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PRETRANSFER_TIME
 */
const CURLINFO_PRETRANSFER_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PRIVATE
 */
const CURLINFO_PRIVATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_REDIRECT_COUNT
 */
const CURLINFO_REDIRECT_COUNT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_REDIRECT_TIME
 */
const CURLINFO_REDIRECT_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_REQUEST_SIZE
 */
const CURLINFO_REQUEST_SIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SIZE_DOWNLOAD
 */
const CURLINFO_SIZE_DOWNLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SIZE_UPLOAD
 */
const CURLINFO_SIZE_UPLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SPEED_DOWNLOAD
 */
const CURLINFO_SPEED_DOWNLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SPEED_UPLOAD
 */
const CURLINFO_SPEED_UPLOAD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SSL_VERIFYRESULT
 */
const CURLINFO_SSL_VERIFYRESULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_STARTTRANSFER_TIME
 */
const CURLINFO_STARTTRANSFER_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_TOTAL_TIME
 */
const CURLINFO_TOTAL_TIME = UNKNOWN;
#if LIBCURL_VERSION_NUM >= 0x074800 /* Available since 7.72.0 */
/**
 * @var int
 * @cvalue CURLINFO_EFFECTIVE_METHOD
 */
const CURLINFO_EFFECTIVE_METHOD = UNKNOWN;
#endif
#if LIBCURL_VERSION_NUM >= 0x075400 /* Available since 7.84.0 */
/**
 * @var int
 * @cvalue CURLINFO_CAPATH
 */
const CURLINFO_CAPATH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_CAINFO
 */
const CURLINFO_CAINFO = UNKNOWN;
#endif
#if LIBCURL_VERSION_NUM >= 0x080c00 /* Available since 8.12.0 */
/**
 * @var int
 * @cvalue CURLINFO_HTTPAUTH_USED
 */
const CURLINFO_HTTPAUTH_USED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PROXYAUTH_USED
 */
const CURLINFO_PROXYAUTH_USED = UNKNOWN;
#endif

/* Other */
/**
 * @var int
 * @cvalue CURLMSG_DONE
 */
const CURLMSG_DONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLVERSION_NOW
 */
const CURLVERSION_NOW = UNKNOWN;

/* Curl Multi Constants */
/**
 * @var int
 * @cvalue CURLM_BAD_EASY_HANDLE
 */
const CURLM_BAD_EASY_HANDLE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLM_BAD_HANDLE
 */
const CURLM_BAD_HANDLE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLM_CALL_MULTI_PERFORM
 */
const CURLM_CALL_MULTI_PERFORM = UNKNOWN;
/**
 * @var int
 * @cvalue CURLM_INTERNAL_ERROR
 */
const CURLM_INTERNAL_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLM_OK
 */
const CURLM_OK = UNKNOWN;
/**
 * @var int
 * @cvalue CURLM_OUT_OF_MEMORY
 */
const CURLM_OUT_OF_MEMORY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLM_ADDED_ALREADY
 */
const CURLM_ADDED_ALREADY = UNKNOWN;

/* Curl proxy constants */
/**
 * @var int
 * @cvalue CURLPROXY_HTTP
 */
const CURLPROXY_HTTP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROXY_SOCKS4
 */
const CURLPROXY_SOCKS4 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROXY_SOCKS5
 */
const CURLPROXY_SOCKS5 = UNKNOWN;

/* Curl Share constants */
/**
 * @var int
 * @cvalue CURLSHOPT_NONE
 */
const CURLSHOPT_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSHOPT_SHARE
 */
const CURLSHOPT_SHARE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSHOPT_UNSHARE
 */
const CURLSHOPT_UNSHARE = UNKNOWN;

/* Curl Http Version constants (CURLOPT_HTTP_VERSION) */
/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_1_0
 */
const CURL_HTTP_VERSION_1_0 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_1_1
 */
const CURL_HTTP_VERSION_1_1 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_NONE
 */
const CURL_HTTP_VERSION_NONE = UNKNOWN;

/* Curl Lock constants */
/**
 * @var int
 * @cvalue CURL_LOCK_DATA_COOKIE
 */
const CURL_LOCK_DATA_COOKIE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_LOCK_DATA_DNS
 */
const CURL_LOCK_DATA_DNS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_LOCK_DATA_SSL_SESSION
 */
const CURL_LOCK_DATA_SSL_SESSION = UNKNOWN;

/* Curl NETRC constants (CURLOPT_NETRC) */
/**
 * @var int
 * @cvalue CURL_NETRC_IGNORED
 */
const CURL_NETRC_IGNORED = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_NETRC_OPTIONAL
 */
const CURL_NETRC_OPTIONAL = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_NETRC_REQUIRED
 */
const CURL_NETRC_REQUIRED = UNKNOWN;

/* Curl SSL Version constants (CURLOPT_SSLVERSION) */
/**
 * @var int
 * @cvalue CURL_SSLVERSION_DEFAULT
 */
const CURL_SSLVERSION_DEFAULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_SSLv2
 */
const CURL_SSLVERSION_SSLv2 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_SSLv3
 */
const CURL_SSLVERSION_SSLv3 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_TLSv1
 */
const CURL_SSLVERSION_TLSv1 = UNKNOWN;

/* Curl TIMECOND constants (CURLOPT_TIMECONDITION) */
/**
 * @var int
 * @cvalue CURL_TIMECOND_IFMODSINCE
 */
const CURL_TIMECOND_IFMODSINCE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_TIMECOND_IFUNMODSINCE
 */
const CURL_TIMECOND_IFUNMODSINCE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_TIMECOND_LASTMOD
 */
const CURL_TIMECOND_LASTMOD = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_TIMECOND_NONE
 */
const CURL_TIMECOND_NONE = UNKNOWN;

/* Curl version constants */
/**
 * @var int
 * @cvalue CURL_VERSION_ASYNCHDNS
 */
const CURL_VERSION_ASYNCHDNS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_CONV
 */
const CURL_VERSION_CONV = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_DEBUG
 */
const CURL_VERSION_DEBUG = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_GSSNEGOTIATE
 */
const CURL_VERSION_GSSNEGOTIATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_IDN
 */
const CURL_VERSION_IDN = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_IPV6
 */
const CURL_VERSION_IPV6 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_KERBEROS4
 */
const CURL_VERSION_KERBEROS4 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_LARGEFILE
 */
const CURL_VERSION_LARGEFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_LIBZ
 */
const CURL_VERSION_LIBZ = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_NTLM
 */
const CURL_VERSION_NTLM = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_SPNEGO
 */
const CURL_VERSION_SPNEGO = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_SSL
 */
const CURL_VERSION_SSL = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_SSPI
 */
const CURL_VERSION_SSPI = UNKNOWN;

/* Available since 7.10.6 */
/**
 * @var int
 * @cvalue CURLOPT_HTTPAUTH
 */
const CURLOPT_HTTPAUTH = UNKNOWN;
/* http authentication options */
/**
 * @var int
 * @cvalue CURLAUTH_ANY
 */
const CURLAUTH_ANY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_ANYSAFE
 */
const CURLAUTH_ANYSAFE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_BASIC
 */
const CURLAUTH_BASIC = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_DIGEST
 */
const CURLAUTH_DIGEST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_GSSNEGOTIATE
 */
const CURLAUTH_GSSNEGOTIATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_NONE
 */
const CURLAUTH_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_NTLM
 */
const CURLAUTH_NTLM = UNKNOWN;

/* Available since 7.10.7 */
/**
 * @var int
 * @cvalue CURLINFO_HTTP_CONNECTCODE
 */
const CURLINFO_HTTP_CONNECTCODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTP_CREATE_MISSING_DIRS
 */
const CURLOPT_FTP_CREATE_MISSING_DIRS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYAUTH
 */
const CURLOPT_PROXYAUTH = UNKNOWN;

/* Available since 7.10.8 */
/**
 * @var int
 * @cvalue CURLE_FILESIZE_EXCEEDED
 */
const CURLE_FILESIZE_EXCEEDED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLE_LDAP_INVALID_URL
 */
const CURLE_LDAP_INVALID_URL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_HTTPAUTH_AVAIL
 */
const CURLINFO_HTTPAUTH_AVAIL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_RESPONSE_CODE
 */
const CURLINFO_RESPONSE_CODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PROXYAUTH_AVAIL
 */
const CURLINFO_PROXYAUTH_AVAIL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SERVER_RESPONSE_TIMEOUT
 * @alias CURLOPT_SERVER_RESPONSE_TIMEOUT
 */
const CURLOPT_FTP_RESPONSE_TIMEOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SERVER_RESPONSE_TIMEOUT
 */
const CURLOPT_SERVER_RESPONSE_TIMEOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_IPRESOLVE
 */
const CURLOPT_IPRESOLVE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAXFILESIZE
 */
const CURLOPT_MAXFILESIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_IPRESOLVE_V4
 */
const CURL_IPRESOLVE_V4 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_IPRESOLVE_V6
 */
const CURL_IPRESOLVE_V6 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_IPRESOLVE_WHATEVER
 */
const CURL_IPRESOLVE_WHATEVER = UNKNOWN;

/* Available since 7.11.0 */
/**
 * @var int
 * @cvalue CURLE_FTP_SSL_FAILED
 */
const CURLE_FTP_SSL_FAILED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_ALL
 */
const CURLFTPSSL_ALL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_CONTROL
 */
const CURLFTPSSL_CONTROL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_NONE
 */
const CURLFTPSSL_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_TRY
 */
const CURLFTPSSL_TRY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTP_SSL
 */
const CURLOPT_FTP_SSL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NETRC_FILE
 */
const CURLOPT_NETRC_FILE = UNKNOWN;

/* Available since 7.11.1 */
/**
 * @var int
 * @cvalue CURLOPT_MAXFILESIZE_LARGE
 */
const CURLOPT_MAXFILESIZE_LARGE = UNKNOWN;

/* Available since 7.11.2 */
/**
 * @var int
 * @cvalue CURLOPT_TCP_NODELAY
 */
const CURLOPT_TCP_NODELAY = UNKNOWN;

/* Available since 7.12.2 */
/**
 * @var int
 * @cvalue CURLFTPAUTH_DEFAULT
 */
const CURLFTPAUTH_DEFAULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPAUTH_SSL
 */
const CURLFTPAUTH_SSL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPAUTH_TLS
 */
const CURLFTPAUTH_TLS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTPSSLAUTH
 */
const CURLOPT_FTPSSLAUTH = UNKNOWN;

/* Available since 7.13.0 */
/**
 * @var int
 * @cvalue CURLOPT_FTP_ACCOUNT
 */
const CURLOPT_FTP_ACCOUNT = UNKNOWN;

/* Available since 7.12.2 */
/**
 * @var int
 * @cvalue CURLINFO_OS_ERRNO
 */
const CURLINFO_OS_ERRNO = UNKNOWN;

/* Available since 7.12.3 */
/**
 * @var int
 * @cvalue CURLINFO_NUM_CONNECTS
 */
const CURLINFO_NUM_CONNECTS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SSL_ENGINES
 */
const CURLINFO_SSL_ENGINES = UNKNOWN;

/* Available since 7.14.1 */
/**
 * @var int
 * @cvalue CURLINFO_COOKIELIST
 */
const CURLINFO_COOKIELIST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_COOKIELIST
 */
const CURLOPT_COOKIELIST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_IGNORE_CONTENT_LENGTH
 */
const CURLOPT_IGNORE_CONTENT_LENGTH = UNKNOWN;

/* Available since 7.15.0 */
/**
 * @var int
 * @cvalue CURLOPT_FTP_SKIP_PASV_IP
 */
const CURLOPT_FTP_SKIP_PASV_IP = UNKNOWN;

/* Available since 7.15.1 */
/**
 * @var int
 * @cvalue CURLOPT_FTP_FILEMETHOD
 */
const CURLOPT_FTP_FILEMETHOD = UNKNOWN;

/* Available since 7.15.2 */
/**
 * @var int
 * @cvalue CURLOPT_CONNECT_ONLY
 */
const CURLOPT_CONNECT_ONLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_LOCALPORT
 */
const CURLOPT_LOCALPORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_LOCALPORTRANGE
 */
const CURLOPT_LOCALPORTRANGE = UNKNOWN;

/* Available since 7.15.3 */
/**
 * @var int
 * @cvalue CURLFTPMETHOD_DEFAULT
 */
const CURLFTPMETHOD_DEFAULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPMETHOD_MULTICWD
 */
const CURLFTPMETHOD_MULTICWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPMETHOD_NOCWD
 */
const CURLFTPMETHOD_NOCWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPMETHOD_SINGLECWD
 */
const CURLFTPMETHOD_SINGLECWD = UNKNOWN;

/* Available since 7.15.4 */
/**
 * @var int
 * @cvalue CURLINFO_FTP_ENTRY_PATH
 */
const CURLINFO_FTP_ENTRY_PATH = UNKNOWN;

/* Available since 7.15.5 */
/**
 * @var int
 * @cvalue CURLOPT_FTP_ALTERNATIVE_TO_USER
 */
const CURLOPT_FTP_ALTERNATIVE_TO_USER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAX_RECV_SPEED_LARGE
 */
const CURLOPT_MAX_RECV_SPEED_LARGE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAX_SEND_SPEED_LARGE
 */
const CURLOPT_MAX_SEND_SPEED_LARGE = UNKNOWN;

/* Available since 7.16.0 */
/**
 * @var int
 * @cvalue CURLE_SSL_CACERT_BADFILE
 */
const CURLE_SSL_CACERT_BADFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_SESSIONID_CACHE
 */
const CURLOPT_SSL_SESSIONID_CACHE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMOPT_PIPELINING
 */
const CURLMOPT_PIPELINING = UNKNOWN;

/* Available since 7.16.1 */
/**
 * @var int
 * @cvalue CURLE_SSH
 */
const CURLE_SSH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTP_SSL_CCC
 */
const CURLOPT_FTP_SSL_CCC = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSH_AUTH_TYPES
 */
const CURLOPT_SSH_AUTH_TYPES = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSH_PRIVATE_KEYFILE
 */
const CURLOPT_SSH_PRIVATE_KEYFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSH_PUBLIC_KEYFILE
 */
const CURLOPT_SSH_PUBLIC_KEYFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_CCC_ACTIVE
 */
const CURLFTPSSL_CCC_ACTIVE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_CCC_NONE
 */
const CURLFTPSSL_CCC_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTPSSL_CCC_PASSIVE
 */
const CURLFTPSSL_CCC_PASSIVE = UNKNOWN;

/* Available since 7.16.2 */
/**
 * @var int
 * @cvalue CURLOPT_CONNECTTIMEOUT_MS
 */
const CURLOPT_CONNECTTIMEOUT_MS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTP_CONTENT_DECODING
 */
const CURLOPT_HTTP_CONTENT_DECODING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HTTP_TRANSFER_DECODING
 */
const CURLOPT_HTTP_TRANSFER_DECODING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TIMEOUT_MS
 */
const CURLOPT_TIMEOUT_MS = UNKNOWN;

/* Available since 7.16.3 */
/**
 * @var int
 * @cvalue CURLMOPT_MAXCONNECTS
 */
const CURLMOPT_MAXCONNECTS = UNKNOWN;

/* Available since 7.16.4 */
/**
 * @var int
 * @cvalue CURLOPT_KRBLEVEL
 */
const CURLOPT_KRBLEVEL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NEW_DIRECTORY_PERMS
 */
const CURLOPT_NEW_DIRECTORY_PERMS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_NEW_FILE_PERMS
 */
const CURLOPT_NEW_FILE_PERMS = UNKNOWN;

/* Available since 7.17.0 */
/**
 * @var int
 * @cvalue CURLOPT_APPEND
 */
const CURLOPT_APPEND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DIRLISTONLY
 */
const CURLOPT_DIRLISTONLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_USE_SSL
 */
const CURLOPT_USE_SSL = UNKNOWN;
/* Curl SSL Constants */
/**
 * @var int
 * @cvalue CURLUSESSL_ALL
 */
const CURLUSESSL_ALL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLUSESSL_CONTROL
 */
const CURLUSESSL_CONTROL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLUSESSL_NONE
 */
const CURLUSESSL_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLUSESSL_TRY
 */
const CURLUSESSL_TRY = UNKNOWN;

/* Available since 7.17.1 */
/**
 * @var int
 * @cvalue CURLOPT_SSH_HOST_PUBLIC_KEY_MD5
 */
const CURLOPT_SSH_HOST_PUBLIC_KEY_MD5 = UNKNOWN;

/* Available since 7.18.0 */
/**
 * @var int
 * @cvalue CURLOPT_PROXY_TRANSFER_MODE
 */
const CURLOPT_PROXY_TRANSFER_MODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPAUSE_ALL
 */
const CURLPAUSE_ALL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPAUSE_CONT
 */
const CURLPAUSE_CONT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPAUSE_RECV
 */
const CURLPAUSE_RECV = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPAUSE_RECV_CONT
 */
const CURLPAUSE_RECV_CONT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPAUSE_SEND
 */
const CURLPAUSE_SEND = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPAUSE_SEND_CONT
 */
const CURLPAUSE_SEND_CONT = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_READFUNC_PAUSE
 */
const CURL_READFUNC_PAUSE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_WRITEFUNC_PAUSE
 */
const CURL_WRITEFUNC_PAUSE = UNKNOWN;

/**
 * @var int
 * @cvalue CURLPROXY_SOCKS4A
 */
const CURLPROXY_SOCKS4A = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROXY_SOCKS5_HOSTNAME
 */
const CURLPROXY_SOCKS5_HOSTNAME = UNKNOWN;

/* Available since 7.18.2 */
/**
 * @var int
 * @cvalue CURLINFO_REDIRECT_URL
 */
const CURLINFO_REDIRECT_URL = UNKNOWN;

/* Available since 7.19.0 */
/**
 * @var int
 * @cvalue CURLINFO_APPCONNECT_TIME
 */
const CURLINFO_APPCONNECT_TIME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PRIMARY_IP
 */
const CURLINFO_PRIMARY_IP = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_ADDRESS_SCOPE
 */
const CURLOPT_ADDRESS_SCOPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CRLFILE
 */
const CURLOPT_CRLFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_ISSUERCERT
 */
const CURLOPT_ISSUERCERT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_KEYPASSWD
 */
const CURLOPT_KEYPASSWD = UNKNOWN;

/**
 * @var int
 * @cvalue CURLSSH_AUTH_ANY
 */
const CURLSSH_AUTH_ANY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSH_AUTH_DEFAULT
 */
const CURLSSH_AUTH_DEFAULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSH_AUTH_HOST
 */
const CURLSSH_AUTH_HOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSH_AUTH_KEYBOARD
 */
const CURLSSH_AUTH_KEYBOARD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSH_AUTH_NONE
 */
const CURLSSH_AUTH_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSH_AUTH_PASSWORD
 */
const CURLSSH_AUTH_PASSWORD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSH_AUTH_PUBLICKEY
 */
const CURLSSH_AUTH_PUBLICKEY = UNKNOWN;

/* Available since 7.19.1 */
/**
 * @var int
 * @cvalue CURLINFO_CERTINFO
 */
const CURLINFO_CERTINFO = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CERTINFO
 */
const CURLOPT_CERTINFO = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PASSWORD
 */
const CURLOPT_PASSWORD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_POSTREDIR
 */
const CURLOPT_POSTREDIR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYPASSWORD
 */
const CURLOPT_PROXYPASSWORD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYUSERNAME
 */
const CURLOPT_PROXYUSERNAME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_USERNAME
 */
const CURLOPT_USERNAME = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_REDIR_POST_301
 */
const CURL_REDIR_POST_301 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_REDIR_POST_302
 */
const CURL_REDIR_POST_302 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_REDIR_POST_ALL
 */
const CURL_REDIR_POST_ALL = UNKNOWN;

/* Available since 7.19.3 */
/**
 * @var int
 * @cvalue CURLAUTH_DIGEST_IE
 */
const CURLAUTH_DIGEST_IE = UNKNOWN;

/* Available since 7.19.4 */
/**
 * @var int
 * @cvalue CURLINFO_CONDITION_UNMET
 */
const CURLINFO_CONDITION_UNMET = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_NOPROXY
 */
const CURLOPT_NOPROXY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROTOCOLS
 */
const CURLOPT_PROTOCOLS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_REDIR_PROTOCOLS
 */
const CURLOPT_REDIR_PROTOCOLS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SOCKS5_GSSAPI_NEC
 */
const CURLOPT_SOCKS5_GSSAPI_NEC = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SOCKS5_GSSAPI_SERVICE
 */
const CURLOPT_SOCKS5_GSSAPI_SERVICE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TFTP_BLKSIZE
 */
const CURLOPT_TFTP_BLKSIZE = UNKNOWN;

/**
 * @var int
 * @cvalue CURLPROTO_ALL
 */
const CURLPROTO_ALL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_DICT
 */
const CURLPROTO_DICT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_FILE
 */
const CURLPROTO_FILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_FTP
 */
const CURLPROTO_FTP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_FTPS
 */
const CURLPROTO_FTPS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_HTTP
 */
const CURLPROTO_HTTP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_HTTPS
 */
const CURLPROTO_HTTPS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_LDAP
 */
const CURLPROTO_LDAP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_LDAPS
 */
const CURLPROTO_LDAPS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_SCP
 */
const CURLPROTO_SCP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_SFTP
 */
const CURLPROTO_SFTP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_TELNET
 */
const CURLPROTO_TELNET = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_TFTP
 */
const CURLPROTO_TFTP = UNKNOWN;

/**
 * @var int
 * @cvalue CURLPROXY_HTTP_1_0
 */
const CURLPROXY_HTTP_1_0 = UNKNOWN;

/**
 * @var int
 * @cvalue CURLFTP_CREATE_DIR
 */
const CURLFTP_CREATE_DIR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTP_CREATE_DIR_NONE
 */
const CURLFTP_CREATE_DIR_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLFTP_CREATE_DIR_RETRY
 */
const CURLFTP_CREATE_DIR_RETRY = UNKNOWN;

/* Available since 7.19.6 */
/**
 * @var int
 * @cvalue CURL_VERSION_CURLDEBUG
 */
const CURL_VERSION_CURLDEBUG = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSH_KNOWNHOSTS
 */
const CURLOPT_SSH_KNOWNHOSTS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLKHMATCH_OK
 */
const CURLKHMATCH_OK = UNKNOWN;
/**
 * @var int
 * @cvalue CURLKHMATCH_MISMATCH
 */
const CURLKHMATCH_MISMATCH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLKHMATCH_MISSING
 */
const CURLKHMATCH_MISSING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLKHMATCH_LAST
 */
const CURLKHMATCH_LAST = UNKNOWN;

/* Available since 7.20.0 */
/**
 * @var int
 * @cvalue CURLINFO_RTSP_CLIENT_CSEQ
 */
const CURLINFO_RTSP_CLIENT_CSEQ = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_RTSP_CSEQ_RECV
 */
const CURLINFO_RTSP_CSEQ_RECV = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_RTSP_SERVER_CSEQ
 */
const CURLINFO_RTSP_SERVER_CSEQ = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_RTSP_SESSION_ID
 */
const CURLINFO_RTSP_SESSION_ID = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FTP_USE_PRET
 */
const CURLOPT_FTP_USE_PRET = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAIL_FROM
 */
const CURLOPT_MAIL_FROM = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_MAIL_RCPT
 */
const CURLOPT_MAIL_RCPT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RTSP_CLIENT_CSEQ
 */
const CURLOPT_RTSP_CLIENT_CSEQ = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RTSP_REQUEST
 */
const CURLOPT_RTSP_REQUEST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RTSP_SERVER_CSEQ
 */
const CURLOPT_RTSP_SERVER_CSEQ = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RTSP_SESSION_ID
 */
const CURLOPT_RTSP_SESSION_ID = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RTSP_STREAM_URI
 */
const CURLOPT_RTSP_STREAM_URI = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RTSP_TRANSPORT
 */
const CURLOPT_RTSP_TRANSPORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_IMAP
 */
const CURLPROTO_IMAP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_IMAPS
 */
const CURLPROTO_IMAPS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_POP3
 */
const CURLPROTO_POP3 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_POP3S
 */
const CURLPROTO_POP3S = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTSP
 */
const CURLPROTO_RTSP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_SMTP
 */
const CURLPROTO_SMTP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_SMTPS
 */
const CURLPROTO_SMTPS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_ANNOUNCE
 */
const CURL_RTSPREQ_ANNOUNCE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_DESCRIBE
 */
const CURL_RTSPREQ_DESCRIBE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_GET_PARAMETER
 */
const CURL_RTSPREQ_GET_PARAMETER = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_OPTIONS
 */
const CURL_RTSPREQ_OPTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_PAUSE
 */
const CURL_RTSPREQ_PAUSE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_PLAY
 */
const CURL_RTSPREQ_PLAY = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_RECEIVE
 */
const CURL_RTSPREQ_RECEIVE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_RECORD
 */
const CURL_RTSPREQ_RECORD = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_SET_PARAMETER
 */
const CURL_RTSPREQ_SET_PARAMETER = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_SETUP
 */
const CURL_RTSPREQ_SETUP = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_RTSPREQ_TEARDOWN
 */
const CURL_RTSPREQ_TEARDOWN = UNKNOWN;

/* Available since 7.21.0 */
/**
 * @var int
 * @cvalue CURLINFO_LOCAL_IP
 */
const CURLINFO_LOCAL_IP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_LOCAL_PORT
 */
const CURLINFO_LOCAL_PORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PRIMARY_PORT
 */
const CURLINFO_PRIMARY_PORT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_FNMATCH_FUNCTION
 */
const CURLOPT_FNMATCH_FUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_WILDCARDMATCH
 */
const CURLOPT_WILDCARDMATCH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTMP
 */
const CURLPROTO_RTMP = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTMPE
 */
const CURLPROTO_RTMPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTMPS
 */
const CURLPROTO_RTMPS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTMPT
 */
const CURLPROTO_RTMPT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTMPTE
 */
const CURLPROTO_RTMPTE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_RTMPTS
 */
const CURLPROTO_RTMPTS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_FNMATCHFUNC_FAIL
 */
const CURL_FNMATCHFUNC_FAIL = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_FNMATCHFUNC_MATCH
 */
const CURL_FNMATCHFUNC_MATCH = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_FNMATCHFUNC_NOMATCH
 */
const CURL_FNMATCHFUNC_NOMATCH = UNKNOWN;

/* Available since 7.21.2 */
/**
 * @var int
 * @cvalue CURLPROTO_GOPHER
 */
const CURLPROTO_GOPHER = UNKNOWN;

/* Available since 7.21.3 */
/**
 * @var int
 * @cvalue CURLAUTH_ONLY
 */
const CURLAUTH_ONLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_RESOLVE
 */
const CURLOPT_RESOLVE = UNKNOWN;

/* Available since 7.21.4 */
/**
 * @var int
 * @cvalue CURLOPT_TLSAUTH_PASSWORD
 */
const CURLOPT_TLSAUTH_PASSWORD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TLSAUTH_TYPE
 */
const CURLOPT_TLSAUTH_TYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TLSAUTH_USERNAME
 */
const CURLOPT_TLSAUTH_USERNAME = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_TLSAUTH_SRP
 */
const CURL_TLSAUTH_SRP = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_TLSAUTH_SRP
 */
const CURL_VERSION_TLSAUTH_SRP = UNKNOWN;

/* Available since 7.21.6 */
/**
 * @var int
 * @cvalue CURLOPT_ACCEPT_ENCODING
 */
const CURLOPT_ACCEPT_ENCODING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TRANSFER_ENCODING
 */
const CURLOPT_TRANSFER_ENCODING = UNKNOWN;

/* Available since 7.22.0 */
/**
 * @var int
 * @cvalue CURLAUTH_NTLM_WB
 */
const CURLAUTH_NTLM_WB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLGSSAPI_DELEGATION_FLAG
 */
const CURLGSSAPI_DELEGATION_FLAG = UNKNOWN;
/**
 * @var int
 * @cvalue CURLGSSAPI_DELEGATION_POLICY_FLAG
 */
const CURLGSSAPI_DELEGATION_POLICY_FLAG = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_GSSAPI_DELEGATION
 */
const CURLOPT_GSSAPI_DELEGATION = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_NTLM_WB
 */
const CURL_VERSION_NTLM_WB = UNKNOWN;

/* Available since 7.24.0 */
/**
 * @var int
 * @cvalue CURLOPT_ACCEPTTIMEOUT_MS
 */
const CURLOPT_ACCEPTTIMEOUT_MS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DNS_SERVERS
 */
const CURLOPT_DNS_SERVERS = UNKNOWN;

/* Available since 7.25.0 */
/**
 * @var int
 * @cvalue CURLOPT_MAIL_AUTH
 */
const CURLOPT_MAIL_AUTH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_OPTIONS
 */
const CURLOPT_SSL_OPTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TCP_KEEPALIVE
 */
const CURLOPT_TCP_KEEPALIVE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TCP_KEEPIDLE
 */
const CURLOPT_TCP_KEEPIDLE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TCP_KEEPINTVL
 */
const CURLOPT_TCP_KEEPINTVL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSLOPT_ALLOW_BEAST
 */
const CURLSSLOPT_ALLOW_BEAST = UNKNOWN;

/* Available since 7.25.1 */
/**
 * @var int
 * @cvalue CURL_REDIR_POST_303
 */
const CURL_REDIR_POST_303 = UNKNOWN;

/* Available since 7.28.0 */
/**
 * @var int
 * @cvalue CURLSSH_AUTH_AGENT
 */
const CURLSSH_AUTH_AGENT = UNKNOWN;

/**
 * @var int
 * @cvalue CURLMOPT_CHUNK_LENGTH_PENALTY_SIZE
 */
const CURLMOPT_CHUNK_LENGTH_PENALTY_SIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMOPT_CONTENT_LENGTH_PENALTY_SIZE
 */
const CURLMOPT_CONTENT_LENGTH_PENALTY_SIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMOPT_MAX_HOST_CONNECTIONS
 */
const CURLMOPT_MAX_HOST_CONNECTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMOPT_MAX_PIPELINE_LENGTH
 */
const CURLMOPT_MAX_PIPELINE_LENGTH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMOPT_MAX_TOTAL_CONNECTIONS
 */
const CURLMOPT_MAX_TOTAL_CONNECTIONS = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_SASL_IR
 */
const CURLOPT_SASL_IR = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_DNS_INTERFACE
 */
const CURLOPT_DNS_INTERFACE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DNS_LOCAL_IP4
 */
const CURLOPT_DNS_LOCAL_IP4 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DNS_LOCAL_IP6
 */
const CURLOPT_DNS_LOCAL_IP6 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_XOAUTH2_BEARER
 */
const CURLOPT_XOAUTH2_BEARER = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_2_0
 */
const CURL_HTTP_VERSION_2_0 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_HTTP2
 */
const CURL_VERSION_HTTP2 = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_LOGIN_OPTIONS
 */
const CURLOPT_LOGIN_OPTIONS = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_SSLVERSION_TLSv1_0
 */
const CURL_SSLVERSION_TLSv1_0 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_TLSv1_1
 */
const CURL_SSLVERSION_TLSv1_1 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_TLSv1_2
 */
const CURL_SSLVERSION_TLSv1_2 = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_EXPECT_100_TIMEOUT_MS
 */
const CURLOPT_EXPECT_100_TIMEOUT_MS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_ENABLE_ALPN
 */
const CURLOPT_SSL_ENABLE_ALPN = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_ENABLE_NPN
 */
const CURLOPT_SSL_ENABLE_NPN = UNKNOWN;

/**
 * @var int
 * @cvalue CURLHEADER_SEPARATE
 */
const CURLHEADER_SEPARATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLHEADER_UNIFIED
 */
const CURLHEADER_UNIFIED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HEADEROPT
 */
const CURLOPT_HEADEROPT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXYHEADER
 */
const CURLOPT_PROXYHEADER = UNKNOWN;

/**
 * @var int
 * @cvalue CURLAUTH_NEGOTIATE
 */
const CURLAUTH_NEGOTIATE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_GSSAPI
 */
const CURL_VERSION_GSSAPI = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_PINNEDPUBLICKEY
 */
const CURLOPT_PINNEDPUBLICKEY = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_UNIX_SOCKET_PATH
 */
const CURLOPT_UNIX_SOCKET_PATH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_SMB
 */
const CURLPROTO_SMB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROTO_SMBS
 */
const CURLPROTO_SMBS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_KERBEROS5
 */
const CURL_VERSION_KERBEROS5 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_UNIX_SOCKETS
 */
const CURL_VERSION_UNIX_SOCKETS = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_SSL_VERIFYSTATUS
 */
const CURLOPT_SSL_VERIFYSTATUS = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_PATH_AS_IS
 */
const CURLOPT_PATH_AS_IS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_FALSESTART
 */
const CURLOPT_SSL_FALSESTART = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_2
 */
const CURL_HTTP_VERSION_2 = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_PIPEWAIT
 */
const CURLOPT_PIPEWAIT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SERVICE_NAME
 */
const CURLOPT_PROXY_SERVICE_NAME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SERVICE_NAME
 */
const CURLOPT_SERVICE_NAME = UNKNOWN;

/**
 * @var int
 * @cvalue CURLPIPE_NOTHING
 */
const CURLPIPE_NOTHING = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPIPE_HTTP1
 */
const CURLPIPE_HTTP1 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPIPE_MULTIPLEX
 */
const CURLPIPE_MULTIPLEX = UNKNOWN;

/**
 * @var int
 * @cvalue CURLSSLOPT_NO_REVOKE
 */
const CURLSSLOPT_NO_REVOKE = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_DEFAULT_PROTOCOL
 */
const CURLOPT_DEFAULT_PROTOCOL = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_STREAM_WEIGHT
 */
const CURLOPT_STREAM_WEIGHT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMOPT_PUSHFUNCTION
 */
const CURLMOPT_PUSHFUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_PUSH_OK
 */
const CURL_PUSH_OK = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_PUSH_DENY
 */
const CURL_PUSH_DENY = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_2TLS
 */
const CURL_HTTP_VERSION_2TLS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_PSL
 */
const CURL_VERSION_PSL = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_TFTP_NO_OPTIONS
 */
const CURLOPT_TFTP_NO_OPTIONS = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_2_PRIOR_KNOWLEDGE
 */
const CURL_HTTP_VERSION_2_PRIOR_KNOWLEDGE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_CONNECT_TO
 */
const CURLOPT_CONNECT_TO = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TCP_FASTOPEN
 */
const CURLOPT_TCP_FASTOPEN = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_HTTP_VERSION
 */
const CURLINFO_HTTP_VERSION = UNKNOWN;

/**
 * @var int
 * @cvalue CURLE_WEIRD_SERVER_REPLY
 */
const CURLE_WEIRD_SERVER_REPLY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_KEEP_SENDING_ON_ERROR
 */
const CURLOPT_KEEP_SENDING_ON_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_SSLVERSION_TLSv1_3
 */
const CURL_SSLVERSION_TLSv1_3 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_HTTPS_PROXY
 */
const CURL_VERSION_HTTPS_PROXY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PROTOCOL
 */
const CURLINFO_PROTOCOL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PROXY_SSL_VERIFYRESULT
 */
const CURLINFO_PROXY_SSL_VERIFYRESULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SCHEME
 */
const CURLINFO_SCHEME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PRE_PROXY
 */
const CURLOPT_PRE_PROXY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_CAINFO
 */
const CURLOPT_PROXY_CAINFO = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_CAPATH
 */
const CURLOPT_PROXY_CAPATH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_CRLFILE
 */
const CURLOPT_PROXY_CRLFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_KEYPASSWD
 */
const CURLOPT_PROXY_KEYPASSWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_PINNEDPUBLICKEY
 */
const CURLOPT_PROXY_PINNEDPUBLICKEY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSL_CIPHER_LIST
 */
const CURLOPT_PROXY_SSL_CIPHER_LIST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSL_OPTIONS
 */
const CURLOPT_PROXY_SSL_OPTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSL_VERIFYHOST
 */
const CURLOPT_PROXY_SSL_VERIFYHOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSL_VERIFYPEER
 */
const CURLOPT_PROXY_SSL_VERIFYPEER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLCERT
 */
const CURLOPT_PROXY_SSLCERT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLCERTTYPE
 */
const CURLOPT_PROXY_SSLCERTTYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLKEY
 */
const CURLOPT_PROXY_SSLKEY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLKEYTYPE
 */
const CURLOPT_PROXY_SSLKEYTYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLVERSION
 */
const CURLOPT_PROXY_SSLVERSION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_TLSAUTH_PASSWORD
 */
const CURLOPT_PROXY_TLSAUTH_PASSWORD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_TLSAUTH_TYPE
 */
const CURLOPT_PROXY_TLSAUTH_TYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_TLSAUTH_USERNAME
 */
const CURLOPT_PROXY_TLSAUTH_USERNAME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPROXY_HTTPS
 */
const CURLPROXY_HTTPS = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_MAX_READ_SIZE
 */
const CURL_MAX_READ_SIZE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_ABSTRACT_UNIX_SOCKET
 */
const CURLOPT_ABSTRACT_UNIX_SOCKET = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_SSLVERSION_MAX_DEFAULT
 */
const CURL_SSLVERSION_MAX_DEFAULT = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_MAX_NONE
 */
const CURL_SSLVERSION_MAX_NONE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_MAX_TLSv1_0
 */
const CURL_SSLVERSION_MAX_TLSv1_0 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_MAX_TLSv1_1
 */
const CURL_SSLVERSION_MAX_TLSv1_1 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_MAX_TLSv1_2
 */
const CURL_SSLVERSION_MAX_TLSv1_2 = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_SSLVERSION_MAX_TLSv1_3
 */
const CURL_SSLVERSION_MAX_TLSv1_3 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SUPPRESS_CONNECT_HEADERS
 */
const CURLOPT_SUPPRESS_CONNECT_HEADERS = UNKNOWN;

/**
 * @var int
 * @cvalue CURLAUTH_GSSAPI
 */
const CURLAUTH_GSSAPI = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_CONTENT_LENGTH_DOWNLOAD_T
 */
const CURLINFO_CONTENT_LENGTH_DOWNLOAD_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_CONTENT_LENGTH_UPLOAD_T
 */
const CURLINFO_CONTENT_LENGTH_UPLOAD_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SIZE_DOWNLOAD_T
 */
const CURLINFO_SIZE_DOWNLOAD_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SIZE_UPLOAD_T
 */
const CURLINFO_SIZE_UPLOAD_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SPEED_DOWNLOAD_T
 */
const CURLINFO_SPEED_DOWNLOAD_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_SPEED_UPLOAD_T
 */
const CURLINFO_SPEED_UPLOAD_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_REQUEST_TARGET
 */
const CURLOPT_REQUEST_TARGET = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SOCKS5_AUTH
 */
const CURLOPT_SOCKS5_AUTH = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_SSH_COMPRESSION
 */
const CURLOPT_SSH_COMPRESSION = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_MULTI_SSL
 */
const CURL_VERSION_MULTI_SSL = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_VERSION_BROTLI
 */
const CURL_VERSION_BROTLI = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_LOCK_DATA_CONNECT
 */
const CURL_LOCK_DATA_CONNECT = UNKNOWN;

/**
 * @var int
 * @cvalue CURLSSH_AUTH_GSSAPI
 */
const CURLSSH_AUTH_GSSAPI = UNKNOWN;

/**
 * @var int
 * @cvalue CURLINFO_FILETIME_T
 */
const CURLINFO_FILETIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HAPPY_EYEBALLS_TIMEOUT_MS
 */
const CURLOPT_HAPPY_EYEBALLS_TIMEOUT_MS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TIMEVALUE_LARGE
 */
const CURLOPT_TIMEVALUE_LARGE = UNKNOWN;

/**
 * @var int
 * @cvalue CURLOPT_DNS_SHUFFLE_ADDRESSES
 */
const CURLOPT_DNS_SHUFFLE_ADDRESSES = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HAPROXYPROTOCOL
 */
const CURLOPT_HAPROXYPROTOCOL = UNKNOWN;

/**
 * @var int
 * @cvalue CURL_LOCK_DATA_PSL
 */
const CURL_LOCK_DATA_PSL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLAUTH_BEARER
 */
const CURLAUTH_BEARER = UNKNOWN;
#if LIBCURL_VERSION_NUM >= 0x080600 /* Available since 8.6.0 */
/**
 * @var int
 * @cvalue CURLINFO_QUEUE_TIME_T
 */
const CURLINFO_QUEUE_TIME_T = UNKNOWN;
#endif
/**
 * @var int
 * @cvalue CURLINFO_APPCONNECT_TIME_T
 */
const CURLINFO_APPCONNECT_TIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_CONNECT_TIME_T
 */
const CURLINFO_CONNECT_TIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_NAMELOOKUP_TIME_T
 */
const CURLINFO_NAMELOOKUP_TIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PRETRANSFER_TIME_T
 */
const CURLINFO_PRETRANSFER_TIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_REDIRECT_TIME_T
 */
const CURLINFO_REDIRECT_TIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_STARTTRANSFER_TIME_T
 */
const CURLINFO_STARTTRANSFER_TIME_T = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_TOTAL_TIME_T
 */
const CURLINFO_TOTAL_TIME_T = UNKNOWN;
#if LIBCURL_VERSION_NUM >= 0x080700 /* Available since 8.7.0 */
/**
 * @var int
 * @cvalue CURLINFO_USED_PROXY
 */
const CURLINFO_USED_PROXY = UNKNOWN;
#endif
#if LIBCURL_VERSION_NUM >= 0x080a00 /* Available since 8.10.0 */
/**
 * @var int
 * @cvalue CURLINFO_POSTTRANSFER_TIME_T
 */
const CURLINFO_POSTTRANSFER_TIME_T = UNKNOWN;
#endif
#if LIBCURL_VERSION_NUM >= 0x080200 /* Available since 8.2.0 */
/**
 * @var int
 * @cvalue CURLINFO_CONN_ID
 */
const CURLINFO_CONN_ID = UNKNOWN;
#endif
/**
 * @var int
 * @cvalue CURLOPT_DISALLOW_USERNAME_IN_URL
 */
const CURLOPT_DISALLOW_USERNAME_IN_URL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_TLS13_CIPHERS
 */
const CURLOPT_PROXY_TLS13_CIPHERS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_TLS13_CIPHERS
 */
const CURLOPT_TLS13_CIPHERS = UNKNOWN;

#if LIBCURL_VERSION_NUM >= 0x073E00 /* Available since 7.62.0 */
/**
 * @var int
 * @cvalue CURLOPT_DOH_URL
 */
const CURLOPT_DOH_URL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_UPKEEP_INTERVAL_MS
 */
const CURLOPT_UPKEEP_INTERVAL_MS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_UPLOAD_BUFFERSIZE
 */
const CURLOPT_UPLOAD_BUFFERSIZE = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074000 /* Available since 7.64.0 */
/**
 * @var int
 * @cvalue CURLOPT_HTTP09_ALLOWED
 */
const CURLOPT_HTTP09_ALLOWED = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074001 /* Available since 7.64.1 */
/**
 * @var int
 * @cvalue CURLALTSVC_H1
 */
const CURLALTSVC_H1 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLALTSVC_H2
 */
const CURLALTSVC_H2 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLALTSVC_H3
 */
const CURLALTSVC_H3 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLALTSVC_READONLYFILE
 */
const CURLALTSVC_READONLYFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_ALTSVC
 */
const CURLOPT_ALTSVC = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_ALTSVC_CTRL
 */
const CURLOPT_ALTSVC_CTRL = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_ALTSVC
 */
const CURL_VERSION_ALTSVC = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074100 /* Available since 7.65.0 */
/**
 * @var int
 * @cvalue CURLOPT_MAXAGE_CONN
 */
const CURLOPT_MAXAGE_CONN = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074200 /* Available since 7.66.0 */
/**
 * @var int
 * @cvalue CURLOPT_SASL_AUTHZID
 */
const CURLOPT_SASL_AUTHZID = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_HTTP3
 */
const CURL_VERSION_HTTP3 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_RETRY_AFTER
 */
const CURLINFO_RETRY_AFTER = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_3
 */
const CURL_HTTP_VERSION_3  = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074300 /* Available since 7.67.0 */
/**
 * @var int
 * @cvalue CURLMOPT_MAX_CONCURRENT_STREAMS
 */
const CURLMOPT_MAX_CONCURRENT_STREAMS = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074400 /* Available since 7.68.0 */
/**
 * @var int
 * @cvalue CURLSSLOPT_NO_PARTIALCHAIN
 */
const CURLSSLOPT_NO_PARTIALCHAIN = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074500 /* Available since 7.69.0 */
/**
 * @var int
 * @cvalue CURLOPT_MAIL_RCPT_ALLLOWFAILS
 */
const CURLOPT_MAIL_RCPT_ALLLOWFAILS = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074600 /* Available since 7.70.0 */
/**
 * @var int
 * @cvalue CURLSSLOPT_REVOKE_BEST_EFFORT
 */
const CURLSSLOPT_REVOKE_BEST_EFFORT = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074700 /* Available since 7.71.0 */
/**
 * @var int
 * @cvalue CURLOPT_ISSUERCERT_BLOB
 */
const CURLOPT_ISSUERCERT_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_ISSUERCERT
 */
const CURLOPT_PROXY_ISSUERCERT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_ISSUERCERT_BLOB
 */
const CURLOPT_PROXY_ISSUERCERT_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLCERT_BLOB
 */
const CURLOPT_PROXY_SSLCERT_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_SSLKEY_BLOB
 */
const CURLOPT_PROXY_SSLKEY_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLCERT_BLOB
 */
const CURLOPT_SSLCERT_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSLKEY_BLOB
 */
const CURLOPT_SSLKEY_BLOB = UNKNOWN;

/**
 * @var int
 * @cvalue CURLPROTO_MQTT
 */
const CURLPROTO_MQTT = UNKNOWN;

/**
 * @var int
 * @cvalue CURLSSLOPT_NATIVE_CA
 */
const CURLSSLOPT_NATIVE_CA = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074800 /* Available since 7.72.0 */
/**
 * @var int
 * @cvalue CURL_VERSION_UNICODE
 */
const CURL_VERSION_UNICODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_ZSTD
 */
const CURL_VERSION_ZSTD = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074900 /* Available since 7.73.0 */
/**
 * @var int
 * @cvalue CURLE_PROXY
 */
const CURLE_PROXY = UNKNOWN;
/**
 * @var int
 * @cvalue CURLINFO_PROXY_ERROR
 */
const CURLINFO_PROXY_ERROR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSL_EC_CURVES
 */
const CURLOPT_SSL_EC_CURVES = UNKNOWN;
#if LIBCURL_VERSION_NUM >= 0x080e00 /* Available since 8.14.0 */
/**
 * @var int
 * @cvalue CURLOPT_SSL_SIGNATURE_ALGORITHMS
 */
const CURLOPT_SSL_SIGNATURE_ALGORITHMS = UNKNOWN;
#endif
/**
 * @var int
 * @cvalue CURLPX_BAD_ADDRESS_TYPE
 */
const CURLPX_BAD_ADDRESS_TYPE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_BAD_VERSION
 */
const CURLPX_BAD_VERSION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_CLOSED
 */
const CURLPX_CLOSED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_GSSAPI
 */
const CURLPX_GSSAPI = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_GSSAPI_PERMSG
 */
const CURLPX_GSSAPI_PERMSG = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_GSSAPI_PROTECTION
 */
const CURLPX_GSSAPI_PROTECTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_IDENTD
 */
const CURLPX_IDENTD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_IDENTD_DIFFER
 */
const CURLPX_IDENTD_DIFFER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_LONG_HOSTNAME
 */
const CURLPX_LONG_HOSTNAME = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_LONG_PASSWD
 */
const CURLPX_LONG_PASSWD = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_LONG_USER
 */
const CURLPX_LONG_USER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_NO_AUTH
 */
const CURLPX_NO_AUTH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_OK
 */
const CURLPX_OK = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_RECV_ADDRESS
 */
const CURLPX_RECV_ADDRESS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_RECV_AUTH
 */
const CURLPX_RECV_AUTH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_RECV_CONNECT
 */
const CURLPX_RECV_CONNECT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_RECV_REQACK
 */
const CURLPX_RECV_REQACK = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_ADDRESS_TYPE_NOT_SUPPORTED
 */
const CURLPX_REPLY_ADDRESS_TYPE_NOT_SUPPORTED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_COMMAND_NOT_SUPPORTED
 */
const CURLPX_REPLY_COMMAND_NOT_SUPPORTED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_CONNECTION_REFUSED
 */
const CURLPX_REPLY_CONNECTION_REFUSED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_GENERAL_SERVER_FAILURE
 */
const CURLPX_REPLY_GENERAL_SERVER_FAILURE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_HOST_UNREACHABLE
 */
const CURLPX_REPLY_HOST_UNREACHABLE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_NETWORK_UNREACHABLE
 */
const CURLPX_REPLY_NETWORK_UNREACHABLE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_NOT_ALLOWED
 */
const CURLPX_REPLY_NOT_ALLOWED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_TTL_EXPIRED
 */
const CURLPX_REPLY_TTL_EXPIRED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REPLY_UNASSIGNED
 */
const CURLPX_REPLY_UNASSIGNED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_REQUEST_FAILED
 */
const CURLPX_REQUEST_FAILED = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_RESOLVE_HOST
 */
const CURLPX_RESOLVE_HOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_SEND_AUTH
 */
const CURLPX_SEND_AUTH = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_SEND_CONNECT
 */
const CURLPX_SEND_CONNECT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_SEND_REQUEST
 */
const CURLPX_SEND_REQUEST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_UNKNOWN_FAIL
 */
const CURLPX_UNKNOWN_FAIL = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_UNKNOWN_MODE
 */
const CURLPX_UNKNOWN_MODE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLPX_USER_REJECTED
 */
const CURLPX_USER_REJECTED = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074a00 /* Available since 7.74.0 */
/**
 * @var int
 * @cvalue CURLHSTS_ENABLE
 */
const CURLHSTS_ENABLE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLHSTS_READONLYFILE
 */
const CURLHSTS_READONLYFILE = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HSTS
 */
const CURLOPT_HSTS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_HSTS_CTRL
 */
const CURLOPT_HSTS_CTRL = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_HSTS
 */
const CURL_VERSION_HSTS = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074b00 /* Available since 7.75.0 */
/**
 * @var int
 * @cvalue CURLAUTH_AWS_SIGV4
 */
const CURLAUTH_AWS_SIGV4 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_AWS_SIGV4
 */
const CURLOPT_AWS_SIGV4 = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074c00 /* Available since 7.76.0 */
/**
 * @var int
 * @cvalue CURLINFO_REFERER
 */
const CURLINFO_REFERER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DOH_SSL_VERIFYHOST
 */
const CURLOPT_DOH_SSL_VERIFYHOST = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DOH_SSL_VERIFYPEER
 */
const CURLOPT_DOH_SSL_VERIFYPEER = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_DOH_SSL_VERIFYSTATUS
 */
const CURLOPT_DOH_SSL_VERIFYSTATUS = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_VERSION_GSASL
 */
const CURL_VERSION_GSASL = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x074d00 /* Available since 7.77.0 */
/**
 * @var int
 * @cvalue CURLOPT_CAINFO_BLOB
 */
const CURLOPT_CAINFO_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PROXY_CAINFO_BLOB
 */
const CURLOPT_PROXY_CAINFO_BLOB = UNKNOWN;
/**
 * @var int
 * @cvalue CURLSSLOPT_AUTO_CLIENT_CERT
 */
const CURLSSLOPT_AUTO_CLIENT_CERT = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075000 /* Available since 7.80.0 */
/**
 * @var int
 * @cvalue CURLOPT_MAXLIFETIME_CONN
 */
const CURLOPT_MAXLIFETIME_CONN = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_SSH_HOST_PUBLIC_KEY_SHA256
 */
const CURLOPT_SSH_HOST_PUBLIC_KEY_SHA256 = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_PREREQFUNCTION
 */
const CURLOPT_PREREQFUNCTION = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_PREREQFUNC_OK
 */
const CURL_PREREQFUNC_OK = UNKNOWN;
/**
 * @var int
 * @cvalue CURL_PREREQFUNC_ABORT
 */
const CURL_PREREQFUNC_ABORT = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075100 /* Available since 7.81.0 */
/**
 * @var int
 * @cvalue CURLOPT_MIME_OPTIONS
 */
const CURLOPT_MIME_OPTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLMIMEOPT_FORMESCAPE
 */
const CURLMIMEOPT_FORMESCAPE = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075400 /* Available since 7.84.0 */
/**
 * @var int
 * @cvalue CURLOPT_SSH_HOSTKEYFUNCTION
 */
const CURLOPT_SSH_HOSTKEYFUNCTION = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075500 /* Available since 7.85.0 */
/**
 * @var int
 * @cvalue CURLOPT_PROTOCOLS_STR
 */
const CURLOPT_PROTOCOLS_STR = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_REDIR_PROTOCOLS_STR
 */
const CURLOPT_REDIR_PROTOCOLS_STR = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075600 /* Available since 7.86.0 */
/**
 * @var int
 * @cvalue CURLOPT_WS_OPTIONS
 */
const CURLOPT_WS_OPTIONS = UNKNOWN;
/**
 * @var int
 * @cvalue CURLWS_RAW_MODE
 */
const CURLWS_RAW_MODE = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075700 /* Available since 7.87.0 */
/**
 * @var int
 * @cvalue CURLOPT_CA_CACHE_TIMEOUT
 */
const CURLOPT_CA_CACHE_TIMEOUT = UNKNOWN;
/**
 * @var int
 * @cvalue CURLOPT_QUICK_EXIT
 */
const CURLOPT_QUICK_EXIT = UNKNOWN;
#endif

#if LIBCURL_VERSION_NUM >= 0x075800 /* Available since 7.88.0 */
/**
 * @var int
 * @cvalue CURL_HTTP_VERSION_3ONLY
 */
const CURL_HTTP_VERSION_3ONLY  = UNKNOWN;
#endif

/**
 * @var int
 * @cvalue CURLOPT_SAFE_UPLOAD
 */
const CURLOPT_SAFE_UPLOAD = UNKNOWN;

/**
 * @strict-properties
 * @not-serializable
 */
final class CurlHandle
{
}

/**
 * @strict-properties
 * @not-serializable
 */
final class CurlMultiHandle
{
}

/**
 * @strict-properties
 * @not-serializable
 */
final class CurlShareHandle
{
}

/**
 * @strict-properties
 * @not-serializable
 */
final class CurlSharePersistentHandle
{
    public readonly array $options;
}

#[\Deprecated(since: '8.5', message: "as it has no effect since PHP 8.0")]
function curl_close(CurlHandle $handle): void {}

/** @refcount 1 */
function curl_copy_handle(CurlHandle $handle): CurlHandle|false {}

function curl_errno(CurlHandle $handle): int {}

/** @refcount 1 */
function curl_error(CurlHandle $handle): string {}

/** @refcount 1 */
function curl_escape(CurlHandle $handle, string $string): string|false {}

/** @refcount 1 */
function curl_unescape(CurlHandle $handle, string $string): string|false {}

function curl_multi_setopt(CurlMultiHandle $multi_handle, int $option, mixed $value): bool {}

function curl_exec(CurlHandle $handle): string|bool {}

/** @refcount 1 */
function curl_file_create(string $filename, ?string $mime_type = null, ?string $posted_filename = null): CURLFile {}

function curl_getinfo(CurlHandle $handle, ?int $option = null): mixed {}

/** @refcount 1 */
function curl_init(?string $url = null): CurlHandle|false {}

#if LIBCURL_VERSION_NUM >= 0x073E00 /* Available since 7.62.0 */
function curl_upkeep(CurlHandle $handle): bool {}
#endif

function curl_multi_add_handle(CurlMultiHandle $multi_handle, CurlHandle $handle): int {}

function curl_multi_get_handles(CurlMultiHandle $multi_handle): array {}

function curl_multi_close(CurlMultiHandle $multi_handle): void {}

function curl_multi_errno(CurlMultiHandle $multi_handle): int {}

/** @param int $still_running */
function curl_multi_exec(CurlMultiHandle $multi_handle, &$still_running): int {}

function curl_multi_getcontent(CurlHandle $handle): ?string {}

/**
 * @param int $queued_messages
 * @return array<string, int|object>|false
 * @refcount 1
 */
function curl_multi_info_read(CurlMultiHandle $multi_handle, &$queued_messages = null): array|false {}

/** @refcount 1 */
function curl_multi_init(): CurlMultiHandle {}

function curl_multi_remove_handle(CurlMultiHandle $multi_handle, CurlHandle $handle): int {}

function curl_multi_select(CurlMultiHandle $multi_handle, float $timeout = 1.0): int {}

/** @refcount 1 */
function curl_multi_strerror(int $error_code): ?string {}

function curl_pause(CurlHandle $handle, int $flags): int {}

function curl_reset(CurlHandle $handle): void {}

function curl_setopt_array(CurlHandle $handle, array $options): bool {}

function curl_setopt(CurlHandle $handle, int $option, mixed $value): bool {}

#[\Deprecated(since: '8.5', message: "as it has no effect since PHP 8.0")]
function curl_share_close(CurlShareHandle $share_handle): void {}

function curl_share_errno(CurlShareHandle $share_handle): int {}

/** @refcount 1 */
function curl_share_init(): CurlShareHandle {}

function curl_share_setopt(CurlShareHandle $share_handle, int $option, mixed $value): bool {}

/** @refcount 1 */
function curl_share_strerror(int $error_code): ?string {}

/** @refcount 1 */
function curl_share_init_persistent(array $share_options): CurlSharePersistentHandle {}

/** @refcount 1 */
function curl_strerror(int $error_code): ?string {}

/**
 * @return array<string, int|string|array>|false
 * @refcount 1
 */
function curl_version(): array|false {}
