EXPORTS
__htmlDefaultSAXHandler
__oldXMLWDcompatibility
__xmlBufferAllocScheme
__xmlDefaultBufferSize
__xmlDefaultSAXHandler
__xmlDefaultSAXLocator
__xmlDeregisterNodeDefaultValue
__xmlDoValidityCheckingDefaultValue
xmlFree DATA
__xmlGenericError
__xmlGenericErrorContext
__xmlGetWarningsDefaultValue
__xmlIndentTreeOutput
xmlIsBaseCharGroup DATA
xmlIsCharGroup DATA
xmlIsCombiningGroup DATA
xmlIsDigitGroup DATA
xmlIsExtenderGroup DATA
xmlIsIdeographicGroup DATA
xmlIsPubidChar_tab DATA
__xmlKeepBlanksDefaultValue
xmlLastError DATA
__xmlLineNumbersDefaultValue
__xmlLoadExtDtdDefaultValue
xmlMalloc DATA
xmlMallocAtomic DATA
xmlMemStrdup DATA
xmlOutputBufferCreateFilenameValue DATA
__xmlParserDebugEntities
xmlParserInputBufferCreateFilenameValue DATA
xmlParserMaxDepth DATA
__xmlParserVersion
__xmlPedanticParserDefaultValue
xmlRealloc DATA
__xmlRegisterNodeDefaultValue
__xmlSaveNoEmptyTags
xmlStringComment DATA
xmlStringText DATA
xmlStringTextNoenc DATA
xmlStructuredError DATA
__xmlSubstituteEntitiesDefaultValue
__xmlTreeIndentString
xmlXPathNAN DATA
xmlXPathNINF DATA
xmlXPathPINF DATA
UTF8ToHtml
UTF8Toisolat1
attribute
attributeDecl
cdataBlock
characters
checkNamespace
comment
elementDecl
endDocument
endElement
entityDecl
externalSubset
getColumnNumber
getEntity
getLineNumber
getNamespace
getParameterEntity
getPublicId
getSystemId
globalNamespace
hasExternalSubset
hasInternalSubset
htmlAttrAllowed
htmlAutoCloseTag
htmlCreateFileParserCtxt
htmlCreateMemoryParserCtxt
htmlCreatePushParserCtxt
htmlCtxtReadDoc
htmlCtxtReadFd
htmlCtxtReadFile
htmlCtxtReadIO
htmlCtxtReadMemory
htmlCtxtReset
htmlCtxtUseOptions
htmlDefaultSAXHandlerInit
htmlDocContentDumpFormatOutput
htmlDocContentDumpOutput
htmlDocDump
htmlDocDumpMemory
htmlElementAllowedHere
htmlElementStatusHere
htmlEncodeEntities
htmlEntityLookup
htmlEntityValueLookup
htmlFreeParserCtxt
htmlGetMetaEncoding
htmlHandleOmittedElem
htmlInitAutoClose
htmlIsAutoClosed
htmlIsBooleanAttr
htmlIsScriptAttribute
htmlNewDoc
htmlNewDocNoDtD
htmlNodeDump
htmlNodeDumpFile
htmlNodeDumpFileFormat
htmlNodeDumpFormatOutput
htmlNodeDumpOutput
htmlNodeStatus
htmlParseCharRef
htmlParseChunk
htmlParseDoc
htmlParseDocument
htmlParseElement
htmlParseEntityRef
htmlParseFile
htmlReadDoc
htmlReadFd
htmlReadFile
htmlReadIO
htmlReadMemory
htmlSAXParseDoc
htmlSAXParseFile
htmlSaveFile
htmlSaveFileEnc
htmlSaveFileFormat
htmlSetMetaEncoding
htmlTagLookup
ignorableWhitespace
initGenericErrorDefaultFunc
inithtmlDefaultSAXHandler
initxmlDefaultSAXHandler
inputPop
inputPush
internalSubset
isStandalone
isolat1ToUTF8
namePop
namePush
namespaceDecl
nodePop
nodePush
notationDecl
processingInstruction
reference
resolveEntity
setDocumentLocator
setNamespace
startDocument
startElement
unparsedEntityDecl
valuePop
valuePush
xmlACatalogAdd
xmlACatalogDump
xmlACatalogRemove
xmlACatalogResolve
xmlACatalogResolvePublic
xmlACatalogResolveSystem
xmlACatalogResolveURI
xmlAddAttributeDecl
xmlAddChild
xmlAddChildList
xmlAddDocEntity
xmlAddDtdEntity
xmlAddElementDecl
xmlAddEncodingAlias
xmlAddID
xmlAddNextSibling
xmlAddNotationDecl
xmlAddPrevSibling
xmlAddRef
xmlAddSibling
xmlAllocOutputBuffer
xmlAllocParserInputBuffer
xmlAttrSerializeTxtContent
xmlAutomataCompile
xmlAutomataGetInitState
xmlAutomataIsDeterminist
xmlAutomataNewAllTrans
xmlAutomataNewCountTrans
xmlAutomataNewCountTrans2
xmlAutomataNewCountedTrans
xmlAutomataNewCounter
xmlAutomataNewCounterTrans
xmlAutomataNewEpsilon
xmlAutomataNewOnceTrans
xmlAutomataNewOnceTrans2
xmlAutomataNewState
xmlAutomataNewTransition
xmlAutomataNewTransition2
xmlAutomataSetFinalState
xmlBoolToText
xmlBufferAdd
xmlBufferAddHead
xmlBufferCCat
xmlBufferCat
xmlBufferContent
xmlBufferCreate
xmlBufferCreateSize
xmlBufferCreateStatic
xmlBufferDump
xmlBufferEmpty
xmlBufferFree
xmlBufferGrow
xmlBufferLength
xmlBufferResize
xmlBufferSetAllocationScheme
xmlBufferShrink
xmlBufferWriteCHAR
xmlBufferWriteChar
xmlBufferWriteQuotedString
xmlBuildQName
xmlBuildRelativeURI
xmlBuildURI
xmlByteConsumed
xmlC14NDocDumpMemory
xmlC14NDocSave
xmlC14NDocSaveTo
xmlC14NExecute
xmlCanonicPath
xmlCatalogAdd
xmlCatalogAddLocal
xmlCatalogCleanup
xmlCatalogConvert
xmlCatalogDump
xmlCatalogFreeLocal
xmlCatalogGetDefaults
xmlCatalogGetPublic
xmlCatalogGetSystem
xmlCatalogIsEmpty
xmlCatalogLocalResolve
xmlCatalogLocalResolveURI
xmlCatalogRemove
xmlCatalogResolve
xmlCatalogResolvePublic
xmlCatalogResolveSystem
xmlCatalogResolveURI
xmlCatalogSetDebug
xmlCatalogSetDefaultPrefer
xmlCatalogSetDefaults
xmlCharEncCloseFunc
xmlCharEncFirstLine
xmlCharEncInFunc
xmlCharEncOutFunc
xmlCharInRange
xmlCharStrdup
xmlCharStrndup
xmlCheckFilename
xmlCheckHTTPInput
xmlCheckLanguageID
xmlCheckUTF8
xmlCheckVersion
xmlCleanupCharEncodingHandlers
xmlCleanupEncodingAliases
xmlCleanupGlobals
xmlCleanupInputCallbacks
xmlCleanupMemory
xmlCleanupOutputCallbacks
xmlCleanupParser
xmlCleanupPredefinedEntities
xmlCleanupThreads
xmlClearNodeInfoSeq
xmlClearParserCtxt
xmlConvertSGMLCatalog
xmlCopyAttributeTable
xmlCopyChar
xmlCopyCharMultiByte
xmlCopyDoc
xmlCopyDtd
xmlCopyElementContent
xmlCopyElementTable
xmlCopyEntitiesTable
xmlCopyEnumeration
xmlCopyError
xmlCopyNamespace
xmlCopyNamespaceList
xmlCopyNode
xmlCopyNodeList
xmlCopyNotationTable
xmlCopyProp
xmlCopyPropList
xmlCreateDocParserCtxt
xmlCreateEntitiesTable
xmlCreateEntityParserCtxt
xmlCreateEnumeration
xmlCreateFileParserCtxt
xmlCreateIOParserCtxt
xmlCreateIntSubset
xmlCreateMemoryParserCtxt
xmlCreatePushParserCtxt
xmlCreateURI
xmlCreateURLParserCtxt
xmlCtxtGetLastError
xmlCtxtReadDoc
xmlCtxtReadFd
xmlCtxtReadFile
xmlCtxtReadIO
xmlCtxtReadMemory
xmlCtxtReset
xmlCtxtResetLastError
xmlCtxtResetPush
xmlCtxtUseOptions
xmlCurrentChar
xmlDebugCheckDocument
xmlDebugDumpAttr
xmlDebugDumpAttrList
xmlDebugDumpDTD
xmlDebugDumpDocument
xmlDebugDumpDocumentHead
xmlDebugDumpEntities
xmlDebugDumpNode
xmlDebugDumpNodeList
xmlDebugDumpOneNode
xmlDebugDumpString
xmlDecodeEntities
xmlDefaultSAXHandlerInit
xmlDelEncodingAlias
xmlDeregisterNodeDefault
xmlDetectCharEncoding
xmlDictCreate
xmlDictCreateSub
xmlDictFree
xmlDictLookup
xmlDictOwns
xmlDictQLookup
xmlDictReference
xmlDictSize
xmlDocCopyNode
xmlDocCopyNodeList
xmlDocDump
xmlDocDumpFormatMemory
xmlDocDumpFormatMemoryEnc
xmlDocDumpMemory
xmlDocDumpMemoryEnc
xmlDocFormatDump
xmlDocGetRootElement
xmlDocSetRootElement
xmlDumpAttributeDecl
xmlDumpAttributeTable
xmlDumpElementDecl
xmlDumpElementTable
xmlDumpEntitiesTable
xmlDumpEntityDecl
xmlDumpNotationDecl
xmlDumpNotationTable
xmlElemDump
xmlEncodeEntities
xmlEncodeEntitiesReentrant
xmlEncodeSpecialChars
xmlFileClose
xmlFileMatch
xmlFileOpen
xmlFileRead
xmlFindCharEncodingHandler
xmlFreeAttributeTable
xmlFreeAutomata
xmlFreeCatalog
xmlFreeDoc
xmlFreeDtd
xmlFreeElementContent
xmlFreeElementTable
xmlFreeEntitiesTable
xmlFreeEnumeration
xmlFreeIDTable
xmlFreeInputStream
xmlFreeMutex
xmlFreeNode
xmlFreeNodeList
xmlFreeNotationTable
xmlFreeNs
xmlFreeNsList
xmlFreeParserCtxt
xmlFreeParserInputBuffer
xmlFreePattern
xmlFreePatternList
xmlFreeProp
xmlFreePropList
xmlFreeRMutex
xmlFreeRefTable
xmlFreeTextReader
xmlFreeTextWriter
xmlFreeURI
xmlFreeValidCtxt
xmlGcMemGet
xmlGcMemSetup
xmlGetBufferAllocationScheme
xmlGetCharEncodingHandler
xmlGetCharEncodingName
xmlGetCompressMode
xmlGetDocCompressMode
xmlGetDocEntity
xmlGetDtdAttrDesc
xmlGetDtdElementDesc
xmlGetDtdEntity
xmlGetDtdNotationDesc
xmlGetDtdQAttrDesc
xmlGetDtdQElementDesc
xmlGetEncodingAlias
xmlGetExternalEntityLoader
xmlGetFeature
xmlGetFeaturesList
xmlGetGlobalState
xmlGetID
xmlGetIntSubset
xmlGetLastChild
xmlGetLastError
xmlGetLineNo
xmlGetNoNsProp
xmlGetNodePath
xmlGetNsList
xmlGetNsProp
xmlGetParameterEntity
xmlGetPredefinedEntity
xmlGetProp
xmlGetRefs
xmlGetThreadId
xmlGetUTF8Char
xmlHandleEntity
xmlHasNsProp
xmlHasProp
xmlHashAddEntry
xmlHashAddEntry2
xmlHashAddEntry3
xmlHashCopy
xmlHashCreate
xmlHashFree
xmlHashLookup
xmlHashLookup2
xmlHashLookup3
xmlHashQLookup
xmlHashQLookup2
xmlHashQLookup3
xmlHashRemoveEntry
xmlHashRemoveEntry2
xmlHashRemoveEntry3
xmlHashScan
xmlHashScan3
xmlHashScanFull
xmlHashScanFull3
xmlHashSize
xmlHashUpdateEntry
xmlHashUpdateEntry2
xmlHashUpdateEntry3
xmlIOFTPClose
xmlIOFTPMatch
xmlIOFTPOpen
xmlIOFTPRead
xmlIOHTTPClose
xmlIOHTTPMatch
xmlIOHTTPOpen
xmlIOHTTPOpenW
xmlIOHTTPRead
xmlIOParseDTD
xmlInitCharEncodingHandlers
xmlInitGlobals
xmlInitMemory
xmlInitNodeInfoSeq
xmlInitParser
xmlInitParserCtxt
xmlInitThreads
xmlInitializeCatalog
xmlInitializeGlobalState
xmlInitializePredefinedEntities
xmlIsBaseChar
xmlIsBlank
xmlIsBlankNode
xmlIsChar
xmlIsCombining
xmlIsDigit
xmlIsExtender
xmlIsID
xmlIsIdeographic
xmlIsLetter
xmlIsMainThread
xmlIsMixedElement
xmlIsPubidChar
xmlIsRef
xmlIsXHTML
xmlKeepBlanksDefault
xmlLineNumbersDefault
xmlLinkGetData
xmlListAppend
xmlListClear
xmlListCopy
xmlListCreate
xmlListDelete
xmlListDup
xmlListEmpty
xmlListEnd
xmlListFront
xmlListInsert
xmlListMerge
xmlListPopBack
xmlListPopFront
xmlListPushBack
xmlListPushFront
xmlListRemoveAll
xmlListRemoveFirst
xmlListRemoveLast
xmlListReverse
xmlListReverseSearch
xmlListReverseWalk
xmlListSearch
xmlListSize
xmlListSort
xmlListWalk
xmlLoadACatalog
xmlLoadCatalog
xmlLoadCatalogs
xmlLoadExternalEntity
xmlLoadSGMLSuperCatalog
xmlLockLibrary
xmlLsCountNode
xmlLsOneNode
xmlMallocAtomicLoc
xmlMallocLoc
xmlMemBlocks
xmlMemDisplay
xmlMemFree
xmlMemGet
xmlMemMalloc
xmlMemRealloc
xmlMemSetup
xmlMemShow
xmlMemStrdupLoc
xmlMemUsed
xmlMemoryDump
xmlMemoryStrdup
xmlModuleClose
xmlModuleFree
xmlModuleOpen
xmlModuleSymbol
xmlMutexLock
xmlMutexUnlock
xmlNamespaceParseNCName
xmlNamespaceParseNSDef
xmlNamespaceParseQName
xmlNanoFTPCheckResponse
xmlNanoFTPCleanup
xmlNanoFTPClose
xmlNanoFTPCloseConnection
xmlNanoFTPConnect
xmlNanoFTPConnectTo
xmlNanoFTPCwd
xmlNanoFTPDele
xmlNanoFTPFreeCtxt
xmlNanoFTPGet
xmlNanoFTPGetConnection
xmlNanoFTPGetResponse
xmlNanoFTPGetSocket
xmlNanoFTPInit
xmlNanoFTPList
xmlNanoFTPNewCtxt
xmlNanoFTPOpen
xmlNanoFTPProxy
xmlNanoFTPQuit
xmlNanoFTPRead
xmlNanoFTPScanProxy
xmlNanoFTPUpdateURL
xmlNanoHTTPAuthHeader
xmlNanoHTTPCleanup
xmlNanoHTTPClose
xmlNanoHTTPContentLength
xmlNanoHTTPEncoding
xmlNanoHTTPFetch
xmlNanoHTTPInit
xmlNanoHTTPMethod
xmlNanoHTTPMethodRedir
xmlNanoHTTPMimeType
xmlNanoHTTPOpen
xmlNanoHTTPOpenRedir
xmlNanoHTTPRead
xmlNanoHTTPRedir
xmlNanoHTTPReturnCode
xmlNanoHTTPSave
xmlNanoHTTPScanProxy
xmlNewAutomata
xmlNewCDataBlock
xmlNewCatalog
xmlNewCharEncodingHandler
xmlNewCharRef
xmlNewChild
xmlNewComment
xmlNewDoc
xmlNewDocComment
xmlNewDocFragment
xmlNewDocNode
xmlNewDocNodeEatName
xmlNewDocPI
xmlNewDocProp
xmlNewDocRawNode
xmlNewDocText
xmlNewDocTextLen
xmlNewDtd
xmlNewElementContent
xmlNewEntityInputStream
xmlNewGlobalNs
xmlNewIOInputStream
xmlNewInputFromFile
xmlNewInputStream
xmlNewMutex
xmlNewNode
xmlNewNodeEatName
xmlNewNs
xmlNewNsProp
xmlNewNsPropEatName
xmlNewPI
xmlNewParserCtxt
xmlNewProp
xmlNewRMutex
xmlNewReference
xmlNewStringInputStream
xmlNewText
xmlNewTextChild
xmlNewTextLen
xmlNewTextReader
xmlNewTextReaderFilename
xmlNewTextWriter
xmlNewTextWriterDoc
xmlNewTextWriterFilename
xmlNewTextWriterMemory
xmlNewTextWriterPushParser
xmlNewTextWriterTree
xmlNewValidCtxt
xmlNextChar
xmlNoNetExternalEntityLoader
xmlNodeAddContent
xmlNodeAddContentLen
xmlNodeBufGetContent
xmlNodeDump
xmlNodeDumpOutput
xmlNodeGetBase
xmlNodeGetContent
xmlNodeGetLang
xmlNodeGetSpacePreserve
xmlNodeIsText
xmlNodeListGetRawString
xmlNodeListGetString
xmlNodeSetBase
xmlNodeSetContent
xmlNodeSetContentLen
xmlNodeSetLang
xmlNodeSetName
xmlNodeSetSpacePreserve
xmlNormalizeURIPath
xmlNormalizeWindowsPath
xmlOutputBufferClose
xmlOutputBufferCreateFd
xmlOutputBufferCreateFile
xmlOutputBufferCreateFilename
xmlOutputBufferCreateFilenameDefault
xmlOutputBufferCreateIO
xmlOutputBufferFlush
xmlOutputBufferWrite
xmlOutputBufferWriteEscape
xmlOutputBufferWriteString
xmlParseAttValue
xmlParseAttribute
xmlParseAttributeListDecl
xmlParseAttributeType
xmlParseBalancedChunkMemory
xmlParseBalancedChunkMemoryRecover
xmlParseCDSect
xmlParseCatalogFile
xmlParseCharData
xmlParseCharEncoding
xmlParseCharRef
xmlParseChunk
xmlParseComment
xmlParseContent
xmlParseCtxtExternalEntity
xmlParseDTD
xmlParseDefaultDecl
xmlParseDoc
xmlParseDocTypeDecl
xmlParseDocument
xmlParseElement
xmlParseElementChildrenContentDecl
xmlParseElementContentDecl
xmlParseElementDecl
xmlParseElementMixedContentDecl
xmlParseEncName
xmlParseEncodingDecl
xmlParseEndTag
xmlParseEntity
xmlParseEntityDecl
xmlParseEntityRef
xmlParseEntityValue
xmlParseEnumeratedType
xmlParseEnumerationType
xmlParseExtParsedEnt
xmlParseExternalEntity
xmlParseExternalID
xmlParseExternalSubset
xmlParseFile
xmlParseInNodeContext
xmlParseMarkupDecl
xmlParseMemory
xmlParseMisc
xmlParseName
xmlParseNamespace
xmlParseNmtoken
xmlParseNotationDecl
xmlParseNotationType
xmlParsePEReference
xmlParsePI
xmlParsePITarget
xmlParsePubidLiteral
xmlParseQuotedString
xmlParseReference
xmlParseSDDecl
xmlParseStartTag
xmlParseSystemLiteral
xmlParseTextDecl
xmlParseURI
xmlParseURIReference
xmlParseVersionInfo
xmlParseVersionNum
xmlParseXMLDecl
xmlParserAddNodeInfo
xmlParserError
xmlParserFindNodeInfo
xmlParserFindNodeInfoIndex
xmlParserGetDirectory
xmlParserHandlePEReference
xmlParserHandleReference
xmlParserInputBufferCreateFd
xmlParserInputBufferCreateFile
xmlParserInputBufferCreateFilename
xmlParserInputBufferCreateFilenameDefault
xmlParserInputBufferCreateIO
xmlParserInputBufferCreateMem
xmlParserInputBufferCreateStatic
xmlParserInputBufferGrow
xmlParserInputBufferPush
xmlParserInputBufferRead
xmlParserInputGrow
xmlParserInputRead
xmlParserInputShrink
xmlParserPrintFileContext
xmlParserPrintFileInfo
xmlParserValidityError
xmlParserValidityWarning
xmlParserWarning
xmlPatternMatch
xmlPatterncompile
xmlPedanticParserDefault
xmlPopInput
xmlPopInputCallbacks
xmlPrintURI
xmlPushInput
xmlRMutexLock
xmlRMutexUnlock
xmlReadDoc
xmlReadFd
xmlReadFile
xmlReadIO
xmlReadMemory
xmlReaderForDoc
xmlReaderForFd
xmlReaderForFile
xmlReaderForIO
xmlReaderForMemory
xmlReaderNewDoc
xmlReaderNewFd
xmlReaderNewFile
xmlReaderNewIO
xmlReaderNewMemory
xmlReaderNewWalker
xmlReaderWalker
xmlReallocLoc
xmlReconciliateNs
xmlRecoverDoc
xmlRecoverFile
xmlRecoverMemory
xmlRegExecPushString
xmlRegExecPushString2
xmlRegFreeExecCtxt
xmlRegFreeRegexp
xmlRegNewExecCtxt
xmlRegexpCompile
xmlRegexpExec
xmlRegexpIsDeterminist
xmlRegexpPrint
xmlRegisterCharEncodingHandler
xmlRegisterDefaultInputCallbacks
xmlRegisterDefaultOutputCallbacks
xmlRegisterHTTPPostCallbacks
xmlRegisterInputCallbacks
xmlRegisterNodeDefault
xmlRegisterOutputCallbacks
xmlRelaxNGDump
xmlRelaxNGDumpTree
xmlRelaxNGFree
xmlRelaxNGFreeParserCtxt
xmlRelaxNGFreeValidCtxt
xmlRelaxNGGetParserErrors
xmlRelaxNGGetValidErrors
xmlRelaxNGInitTypes
xmlRelaxNGNewDocParserCtxt
xmlRelaxNGNewMemParserCtxt
xmlRelaxNGNewParserCtxt
xmlRelaxNGNewValidCtxt
xmlRelaxNGParse
xmlRelaxNGSetParserErrors
xmlRelaxNGSetValidErrors
xmlRelaxNGValidateDoc
xmlRelaxNGValidateFullElement
xmlRelaxNGValidatePopElement
xmlRelaxNGValidatePushCData
xmlRelaxNGValidatePushElement
xmlRelaxParserSetFlag
xmlRemoveID
xmlRemoveProp
xmlRemoveRef
xmlReplaceNode
xmlResetError
xmlResetLastError
xmlSAX2AttributeDecl
xmlSAX2CDataBlock
xmlSAX2Characters
xmlSAX2Comment
xmlSAX2ElementDecl
xmlSAX2EndDocument
xmlSAX2EndElement
xmlSAX2EndElementNs
xmlSAX2EntityDecl
xmlSAX2ExternalSubset
xmlSAX2GetColumnNumber
xmlSAX2GetEntity
xmlSAX2GetLineNumber
xmlSAX2GetParameterEntity
xmlSAX2GetPublicId
xmlSAX2GetSystemId
xmlSAX2HasExternalSubset
xmlSAX2HasInternalSubset
xmlSAX2IgnorableWhitespace
xmlSAX2InitDefaultSAXHandler
xmlSAX2InitHtmlDefaultSAXHandler
xmlSAX2InternalSubset
xmlSAX2IsStandalone
xmlSAX2NotationDecl
xmlSAX2ProcessingInstruction
xmlSAX2Reference
xmlSAX2ResolveEntity
xmlSAX2SetDocumentLocator
xmlSAX2StartDocument
xmlSAX2StartElement
xmlSAX2StartElementNs
xmlSAX2UnparsedEntityDecl
xmlSAXDefaultVersion
xmlSAXParseDTD
xmlSAXParseDoc
xmlSAXParseEntity
xmlSAXParseFile
xmlSAXParseFileWithData
xmlSAXParseMemory
xmlSAXParseMemoryWithData
xmlSAXUserParseFile
xmlSAXUserParseMemory
xmlSAXVersion
xmlSaveClose
xmlSaveDoc
xmlSaveFile
xmlSaveFileEnc
xmlSaveFileTo
xmlSaveFlush
xmlSaveFormatFile
xmlSaveFormatFileEnc
xmlSaveFormatFileTo
xmlSaveSetAttrEscape
xmlSaveSetEscape
xmlSaveToFd
xmlSaveToFilename
xmlSaveToIO
xmlSaveTree
xmlSaveUri
xmlScanName
xmlSchemaCheckFacet
xmlSchemaCleanupTypes
xmlSchemaCollapseString
xmlSchemaCompareValues
xmlSchemaDump
xmlSchemaFree
xmlSchemaFreeFacet
xmlSchemaFreeParserCtxt
xmlSchemaFreeType
xmlSchemaFreeValidCtxt
xmlSchemaFreeValue
xmlSchemaFreeWildcard
xmlSchemaGetBuiltInListSimpleTypeItemType
xmlSchemaGetBuiltInType
xmlSchemaGetFacetValueAsULong
xmlSchemaGetParserErrors
xmlSchemaGetPredefinedType
xmlSchemaGetValidErrors
xmlSchemaInitTypes
xmlSchemaIsBuiltInTypeFacet
xmlSchemaNewDocParserCtxt
xmlSchemaNewFacet
xmlSchemaNewMemParserCtxt
xmlSchemaNewParserCtxt
xmlSchemaNewValidCtxt
xmlSchemaParse
xmlSchemaSetParserErrors
xmlSchemaSetValidErrors
xmlSchemaSetValidOptions
xmlSchemaValPredefTypeNode
xmlSchemaValPredefTypeNodeNoNorm
xmlSchemaValidCtxtGetOptions
xmlSchemaValidateDoc
xmlSchemaValidateFacet
xmlSchemaValidateLengthFacet
xmlSchemaValidateListSimpleTypeFacet
xmlSchemaValidateOneElement
xmlSchemaValidatePredefinedType
xmlSchemaValidateStream
xmlSearchNs
xmlSearchNsByHref
xmlSetBufferAllocationScheme
xmlSetCompressMode
xmlSetDocCompressMode
xmlSetEntityReferenceFunc
xmlSetExternalEntityLoader
xmlSetFeature
xmlSetGenericErrorFunc
xmlSetListDoc
xmlSetNs
xmlSetNsProp
xmlSetProp
xmlSetStructuredErrorFunc
xmlSetTreeDoc
xmlSetupParserForBuffer
xmlShell
xmlShellBase
xmlShellCat
xmlShellDir
xmlShellDu
xmlShellList
xmlShellLoad
xmlShellPrintNode
xmlShellPrintXPathError
xmlShellPrintXPathResult
xmlShellPwd
xmlShellSave
xmlShellValidate
xmlShellWrite
xmlSkipBlankChars
xmlSnprintfElementContent
xmlSplitQName
xmlSplitQName2
xmlSplitQName3
xmlSprintfElementContent
xmlStopParser
xmlStrEqual
xmlStrPrintf
xmlStrQEqual
xmlStrVPrintf
xmlStrcasecmp
xmlStrcasestr
xmlStrcat
xmlStrchr
xmlStrcmp
xmlStrdup
xmlStringCurrentChar
xmlStringDecodeEntities
xmlStringGetNodeList
xmlStringLenDecodeEntities
xmlStringLenGetNodeList
xmlStrlen
xmlStrncasecmp
xmlStrncat
xmlStrncatNew
xmlStrncmp
xmlStrndup
xmlStrstr
xmlStrsub
xmlSubstituteEntitiesDefault
xmlSwitchEncoding
xmlSwitchInputEncoding
xmlSwitchToEncoding
xmlTextConcat
xmlTextMerge
xmlTextReaderAttributeCount
xmlTextReaderBaseUri
xmlTextReaderClose
xmlTextReaderConstBaseUri
xmlTextReaderConstEncoding
xmlTextReaderConstLocalName
xmlTextReaderConstName
xmlTextReaderConstNamespaceUri
xmlTextReaderConstPrefix
xmlTextReaderConstString
xmlTextReaderConstValue
xmlTextReaderConstXmlLang
xmlTextReaderConstXmlVersion
xmlTextReaderCurrentDoc
xmlTextReaderCurrentNode
xmlTextReaderDepth
xmlTextReaderExpand
xmlTextReaderGetAttribute
xmlTextReaderGetAttributeNo
xmlTextReaderGetAttributeNs
xmlTextReaderGetErrorHandler
xmlTextReaderGetParserProp
xmlTextReaderGetRemainder
xmlTextReaderHasAttributes
xmlTextReaderHasValue
xmlTextReaderIsDefault
xmlTextReaderIsEmptyElement
xmlTextReaderIsNamespaceDecl
xmlTextReaderIsValid
xmlTextReaderLocalName
xmlTextReaderLocatorBaseURI
xmlTextReaderLocatorLineNumber
xmlTextReaderLookupNamespace
xmlTextReaderMoveToAttribute
xmlTextReaderMoveToAttributeNo
xmlTextReaderMoveToAttributeNs
xmlTextReaderMoveToElement
xmlTextReaderMoveToFirstAttribute
xmlTextReaderMoveToNextAttribute
xmlTextReaderName
xmlTextReaderNamespaceUri
xmlTextReaderNext
xmlTextReaderNextSibling
xmlTextReaderNodeType
xmlTextReaderNormalization
xmlTextReaderPrefix
xmlTextReaderPreserve
xmlTextReaderPreservePattern
xmlTextReaderQuoteChar
xmlTextReaderRead
xmlTextReaderReadAttributeValue
xmlTextReaderReadInnerXml
xmlTextReaderReadOuterXml
xmlTextReaderReadState
xmlTextReaderReadString
xmlTextReaderRelaxNGSetSchema
xmlTextReaderRelaxNGValidate
xmlTextReaderSchemaValidate
xmlTextReaderSetErrorHandler
xmlTextReaderSetParserProp
xmlTextReaderSetStructuredErrorHandler
xmlTextReaderSetup
xmlTextReaderStandalone
xmlTextReaderValue
xmlTextReaderXmlLang
xmlTextWriterEndAttribute
xmlTextWriterEndCDATA
xmlTextWriterEndComment
xmlTextWriterEndDTD
xmlTextWriterEndDTDAttlist
xmlTextWriterEndDTDElement
xmlTextWriterEndDTDEntity
xmlTextWriterEndDocument
xmlTextWriterEndElement
xmlTextWriterEndPI
xmlTextWriterFlush
xmlTextWriterFullEndElement
xmlTextWriterSetIndent
xmlTextWriterSetIndentString
xmlTextWriterStartAttribute
xmlTextWriterStartAttributeNS
xmlTextWriterStartCDATA
xmlTextWriterStartComment
xmlTextWriterStartDTD
xmlTextWriterStartDTDAttlist
xmlTextWriterStartDTDElement
xmlTextWriterStartDTDEntity
xmlTextWriterStartDocument
xmlTextWriterStartElement
xmlTextWriterStartElementNS
xmlTextWriterStartPI
xmlTextWriterWriteAttribute
xmlTextWriterWriteAttributeNS
xmlTextWriterWriteBase64
xmlTextWriterWriteBinHex
xmlTextWriterWriteCDATA
xmlTextWriterWriteComment
xmlTextWriterWriteDTD
xmlTextWriterWriteDTDAttlist
xmlTextWriterWriteDTDElement
xmlTextWriterWriteDTDEntity
xmlTextWriterWriteDTDExternalEntity
xmlTextWriterWriteDTDExternalEntityContents
xmlTextWriterWriteDTDInternalEntity
xmlTextWriterWriteDTDNotation
xmlTextWriterWriteElement
xmlTextWriterWriteElementNS
xmlTextWriterWriteFormatAttribute
xmlTextWriterWriteFormatAttributeNS
xmlTextWriterWriteFormatCDATA
xmlTextWriterWriteFormatComment
xmlTextWriterWriteFormatDTD
xmlTextWriterWriteFormatDTDAttlist
xmlTextWriterWriteFormatDTDElement
xmlTextWriterWriteFormatDTDInternalEntity
xmlTextWriterWriteFormatElement
xmlTextWriterWriteFormatElementNS
xmlTextWriterWriteFormatPI
xmlTextWriterWriteFormatRaw
xmlTextWriterWriteFormatString
xmlTextWriterWritePI
xmlTextWriterWriteRaw
xmlTextWriterWriteRawLen
xmlTextWriterWriteString
xmlTextWriterWriteVFormatAttribute
xmlTextWriterWriteVFormatAttributeNS
xmlTextWriterWriteVFormatCDATA
xmlTextWriterWriteVFormatComment
xmlTextWriterWriteVFormatDTD
xmlTextWriterWriteVFormatDTDAttlist
xmlTextWriterWriteVFormatDTDElement
xmlTextWriterWriteVFormatDTDInternalEntity
xmlTextWriterWriteVFormatElement
xmlTextWriterWriteVFormatElementNS
xmlTextWriterWriteVFormatPI
xmlTextWriterWriteVFormatRaw
xmlTextWriterWriteVFormatString
xmlThrDefBufferAllocScheme
xmlThrDefDefaultBufferSize
xmlThrDefDeregisterNodeDefault
xmlThrDefDoValidityCheckingDefaultValue
xmlThrDefGetWarningsDefaultValue
xmlThrDefIndentTreeOutput
xmlThrDefKeepBlanksDefaultValue
xmlThrDefLineNumbersDefaultValue
xmlThrDefLoadExtDtdDefaultValue
xmlThrDefOutputBufferCreateFilenameDefault
xmlThrDefParserDebugEntities
xmlThrDefParserInputBufferCreateFilenameDefault
xmlThrDefPedanticParserDefaultValue
xmlThrDefRegisterNodeDefault
xmlThrDefSaveNoEmptyTags
xmlThrDefSetGenericErrorFunc
xmlThrDefSetStructuredErrorFunc
xmlThrDefSubstituteEntitiesDefaultValue
xmlThrDefTreeIndentString
xmlUCSIsAegeanNumbers
xmlUCSIsAlphabeticPresentationForms
xmlUCSIsArabic
xmlUCSIsArabicPresentationFormsA
xmlUCSIsArabicPresentationFormsB
xmlUCSIsArmenian
xmlUCSIsArrows
xmlUCSIsBasicLatin
xmlUCSIsBengali
xmlUCSIsBlock
xmlUCSIsBlockElements
xmlUCSIsBopomofo
xmlUCSIsBopomofoExtended
xmlUCSIsBoxDrawing
xmlUCSIsBraillePatterns
xmlUCSIsBuhid
xmlUCSIsByzantineMusicalSymbols
xmlUCSIsCJKCompatibility
xmlUCSIsCJKCompatibilityForms
xmlUCSIsCJKCompatibilityIdeographs
xmlUCSIsCJKCompatibilityIdeographsSupplement
xmlUCSIsCJKRadicalsSupplement
xmlUCSIsCJKSymbolsandPunctuation
xmlUCSIsCJKUnifiedIdeographs
xmlUCSIsCJKUnifiedIdeographsExtensionA
xmlUCSIsCJKUnifiedIdeographsExtensionB
xmlUCSIsCat
xmlUCSIsCatC
xmlUCSIsCatCc
xmlUCSIsCatCf
xmlUCSIsCatCo
xmlUCSIsCatCs
xmlUCSIsCatL
xmlUCSIsCatLl
xmlUCSIsCatLm
xmlUCSIsCatLo
xmlUCSIsCatLt
xmlUCSIsCatLu
xmlUCSIsCatM
xmlUCSIsCatMc
xmlUCSIsCatMe
xmlUCSIsCatMn
xmlUCSIsCatN
xmlUCSIsCatNd
xmlUCSIsCatNl
xmlUCSIsCatNo
xmlUCSIsCatP
xmlUCSIsCatPc
xmlUCSIsCatPd
xmlUCSIsCatPe
xmlUCSIsCatPf
xmlUCSIsCatPi
xmlUCSIsCatPo
xmlUCSIsCatPs
xmlUCSIsCatS
xmlUCSIsCatSc
xmlUCSIsCatSk
xmlUCSIsCatSm
xmlUCSIsCatSo
xmlUCSIsCatZ
xmlUCSIsCatZl
xmlUCSIsCatZp
xmlUCSIsCatZs
xmlUCSIsCherokee
xmlUCSIsCombiningDiacriticalMarks
xmlUCSIsCombiningDiacriticalMarksforSymbols
xmlUCSIsCombiningHalfMarks
xmlUCSIsCombiningMarksforSymbols
xmlUCSIsControlPictures
xmlUCSIsCurrencySymbols
xmlUCSIsCypriotSyllabary
xmlUCSIsCyrillic
xmlUCSIsCyrillicSupplement
xmlUCSIsDeseret
xmlUCSIsDevanagari
xmlUCSIsDingbats
xmlUCSIsEnclosedAlphanumerics
xmlUCSIsEnclosedCJKLettersandMonths
xmlUCSIsEthiopic
xmlUCSIsGeneralPunctuation
xmlUCSIsGeometricShapes
xmlUCSIsGeorgian
xmlUCSIsGothic
xmlUCSIsGreek
xmlUCSIsGreekExtended
xmlUCSIsGreekandCoptic
xmlUCSIsGujarati
xmlUCSIsGurmukhi
xmlUCSIsHalfwidthandFullwidthForms
xmlUCSIsHangulCompatibilityJamo
xmlUCSIsHangulJamo
xmlUCSIsHangulSyllables
xmlUCSIsHanunoo
xmlUCSIsHebrew
xmlUCSIsHighPrivateUseSurrogates
xmlUCSIsHighSurrogates
xmlUCSIsHiragana
xmlUCSIsIPAExtensions
xmlUCSIsIdeographicDescriptionCharacters
xmlUCSIsKanbun
xmlUCSIsKangxiRadicals
xmlUCSIsKannada
xmlUCSIsKatakana
xmlUCSIsKatakanaPhoneticExtensions
xmlUCSIsKhmer
xmlUCSIsKhmerSymbols
xmlUCSIsLao
xmlUCSIsLatin1Supplement
xmlUCSIsLatinExtendedA
xmlUCSIsLatinExtendedAdditional
xmlUCSIsLatinExtendedB
xmlUCSIsLetterlikeSymbols
xmlUCSIsLimbu
xmlUCSIsLinearBIdeograms
xmlUCSIsLinearBSyllabary
xmlUCSIsLowSurrogates
xmlUCSIsMalayalam
xmlUCSIsMathematicalAlphanumericSymbols
xmlUCSIsMathematicalOperators
xmlUCSIsMiscellaneousMathematicalSymbolsA
xmlUCSIsMiscellaneousMathematicalSymbolsB
xmlUCSIsMiscellaneousSymbols
xmlUCSIsMiscellaneousSymbolsandArrows
xmlUCSIsMiscellaneousTechnical
xmlUCSIsMongolian
xmlUCSIsMusicalSymbols
xmlUCSIsMyanmar
xmlUCSIsNumberForms
xmlUCSIsOgham
xmlUCSIsOldItalic
xmlUCSIsOpticalCharacterRecognition
xmlUCSIsOriya
xmlUCSIsOsmanya
xmlUCSIsPhoneticExtensions
xmlUCSIsPrivateUse
xmlUCSIsPrivateUseArea
xmlUCSIsRunic
xmlUCSIsShavian
xmlUCSIsSinhala
xmlUCSIsSmallFormVariants
xmlUCSIsSpacingModifierLetters
xmlUCSIsSpecials
xmlUCSIsSuperscriptsandSubscripts
xmlUCSIsSupplementalArrowsA
xmlUCSIsSupplementalArrowsB
xmlUCSIsSupplementalMathematicalOperators
xmlUCSIsSupplementaryPrivateUseAreaA
xmlUCSIsSupplementaryPrivateUseAreaB
xmlUCSIsSyriac
xmlUCSIsTagalog
xmlUCSIsTagbanwa
xmlUCSIsTags
xmlUCSIsTaiLe
xmlUCSIsTaiXuanJingSymbols
xmlUCSIsTamil
xmlUCSIsTelugu
xmlUCSIsThaana
xmlUCSIsThai
xmlUCSIsTibetan
xmlUCSIsUgaritic
xmlUCSIsUnifiedCanadianAboriginalSyllabics
xmlUCSIsVariationSelectors
xmlUCSIsVariationSelectorsSupplement
xmlUCSIsYiRadicals
xmlUCSIsYiSyllables
xmlUCSIsYijingHexagramSymbols
xmlURIEscape
xmlURIEscapeStr
xmlURIUnescapeString
xmlUTF8Charcmp
xmlUTF8Size
xmlUTF8Strlen
xmlUTF8Strloc
xmlUTF8Strndup
xmlUTF8Strpos
xmlUTF8Strsize
xmlUTF8Strsub
xmlUnlinkNode
xmlUnlockLibrary
xmlUnsetNsProp
xmlUnsetProp
xmlValidBuildContentModel
xmlValidCtxtNormalizeAttributeValue
xmlValidGetPotentialChildren
xmlValidGetValidElements
xmlValidNormalizeAttributeValue
xmlValidateAttributeDecl
xmlValidateAttributeValue
xmlValidateDocument
xmlValidateDocumentFinal
xmlValidateDtd
xmlValidateDtdFinal
xmlValidateElement
xmlValidateElementDecl
xmlValidateNCName
xmlValidateNMToken
xmlValidateName
xmlValidateNameValue
xmlValidateNamesValue
xmlValidateNmtokenValue
xmlValidateNmtokensValue
xmlValidateNotationDecl
xmlValidateNotationUse
xmlValidateOneAttribute
xmlValidateOneElement
xmlValidateOneNamespace
xmlValidatePopElement
xmlValidatePushCData
xmlValidatePushElement
xmlValidateQName
xmlValidateRoot
xmlXIncludeFreeContext
xmlXIncludeNewContext
xmlXIncludeProcess
xmlXIncludeProcessFlags
xmlXIncludeProcessNode
xmlXIncludeProcessTree
xmlXIncludeProcessTreeFlags
xmlXIncludeSetFlags
xmlXPathAddValues
xmlXPathBooleanFunction
xmlXPathCastBooleanToNumber
xmlXPathCastBooleanToString
xmlXPathCastNodeSetToBoolean
xmlXPathCastNodeSetToNumber
xmlXPathCastNodeSetToString
xmlXPathCastNodeToNumber
xmlXPathCastNodeToString
xmlXPathCastNumberToBoolean
xmlXPathCastNumberToString
xmlXPathCastStringToBoolean
xmlXPathCastStringToNumber
xmlXPathCastToBoolean
xmlXPathCastToNumber
xmlXPathCastToString
xmlXPathCeilingFunction
xmlXPathCmpNodes
xmlXPathCompareValues
xmlXPathCompile
xmlXPathCompiledEval
xmlXPathCompiledEvalToBoolean
xmlXPathConcatFunction
xmlXPathContainsFunction
xmlXPathContextSetCache
xmlXPathConvertBoolean
xmlXPathConvertNumber
xmlXPathConvertString
xmlXPathCountFunction
xmlXPathCtxtCompile
xmlXPathDebugDumpCompExpr
xmlXPathDebugDumpObject
xmlXPathDifference
xmlXPathDistinct
xmlXPathDistinctSorted
xmlXPathDivValues
xmlXPathEqualValues
xmlXPathErr
xmlXPathEval
xmlXPathEvalExpr
xmlXPathEvalExpression
xmlXPathEvalPredicate
xmlXPathEvaluatePredicateResult
xmlXPathFalseFunction
xmlXPathFloorFunction
xmlXPathFreeCompExpr
xmlXPathFreeContext
xmlXPathFreeNodeSet
xmlXPathFreeNodeSetList
xmlXPathFreeObject
xmlXPathFreeParserContext
xmlXPathFunctionLookup
xmlXPathFunctionLookupNS
xmlXPathHasSameNodes
xmlXPathIdFunction
xmlXPathInit
xmlXPathIntersection
xmlXPathIsInf
xmlXPathIsNaN
xmlXPathIsNodeType
xmlXPathLangFunction
xmlXPathLastFunction
xmlXPathLeading
xmlXPathLeadingSorted
xmlXPathLocalNameFunction
xmlXPathModValues
xmlXPathMultValues
xmlXPathNamespaceURIFunction
xmlXPathNewBoolean
xmlXPathNewCString
xmlXPathNewContext
xmlXPathNewFloat
xmlXPathNewNodeSet
xmlXPathNewNodeSetList
xmlXPathNewParserContext
xmlXPathNewString
xmlXPathNewValueTree
xmlXPathNextAncestor
xmlXPathNextAncestorOrSelf
xmlXPathNextAttribute
xmlXPathNextChild
xmlXPathNextDescendant
xmlXPathNextDescendantOrSelf
xmlXPathNextFollowing
xmlXPathNextFollowingSibling
xmlXPathNextNamespace
xmlXPathNextParent
xmlXPathNextPreceding
xmlXPathNextPrecedingSibling
xmlXPathNextSelf
xmlXPathNodeLeading
xmlXPathNodeLeadingSorted
xmlXPathNodeSetAdd
xmlXPathNodeSetAddNs
xmlXPathNodeSetAddUnique
xmlXPathNodeSetContains
xmlXPathNodeSetCreate
xmlXPathNodeSetDel
xmlXPathNodeSetFreeNs
xmlXPathNodeSetMerge
xmlXPathNodeSetRemove
xmlXPathNodeSetSort
xmlXPathNodeTrailing
xmlXPathNodeTrailingSorted
xmlXPathNormalizeFunction
xmlXPathNotEqualValues
xmlXPathNotFunction
xmlXPathNsLookup
xmlXPathNumberFunction
xmlXPathObjectCopy
xmlXPathOrderDocElems
xmlXPathParseNCName
xmlXPathParseName
xmlXPathPopBoolean
xmlXPathPopExternal
xmlXPathPopNodeSet
xmlXPathPopNumber
xmlXPathPopString
xmlXPathPositionFunction
xmlXPathRegisterAllFunctions
xmlXPathRegisterFunc
xmlXPathRegisterFuncLookup
xmlXPathRegisterFuncNS
xmlXPathRegisterNs
xmlXPathRegisterVariable
xmlXPathRegisterVariableLookup
xmlXPathRegisterVariableNS
xmlXPathRegisteredFuncsCleanup
xmlXPathRegisteredNsCleanup
xmlXPathRegisteredVariablesCleanup
xmlXPathRoot
xmlXPathRoundFunction
xmlXPathStartsWithFunction
xmlXPathStringEvalNumber
xmlXPathStringFunction
xmlXPathStringLengthFunction
xmlXPathSubValues
xmlXPathSubstringAfterFunction
xmlXPathSubstringBeforeFunction
xmlXPathSubstringFunction
xmlXPathSumFunction
xmlXPathTrailing
xmlXPathTrailingSorted
xmlXPathTranslateFunction
xmlXPathTrueFunction
xmlXPathValueFlipSign
xmlXPathVariableLookup
xmlXPathVariableLookupNS
xmlXPathWrapCString
xmlXPathWrapExternal
xmlXPathWrapNodeSet
xmlXPathWrapString
xmlXPatherror
xmlXPtrEval
xmlXPtrNewContext
xmlBufContent
xmlBufUse
xmlBufferDetach
