<?php

/** @generate-class-entries */

#if SODIUM_LIBRARY_VERSION_MAJOR > 9 || (SODIUM_LIBRARY_VERSION_MAJOR == 9 && SODIUM_LIBRARY_VERSION_MINOR >= 6)
const PASSWORD_ARGON2I = "argon2i";
const PASSWORD_ARGON2ID = "argon2id";
/**
 * @var int
 * @cvalue PHP_SODIUM_PWHASH_MEMLIMIT
 */
const PASSWORD_ARGON2_DEFAULT_MEMORY_COST = UNKNOWN;
/**
 * @var int
 * @cvalue PHP_SODIUM_PWHASH_OPSLIMIT
 */
const PASSWORD_ARGON2_DEFAULT_TIME_COST = UNKNOWN;
/**
 * @var int
 * @cvalue PHP_SODIUM_PWHASH_THREADS
 */
const PASSWORD_ARGON2_DEFAULT_THREADS = UNKNOWN;
const PASSWORD_ARGON2_PROVIDER = "sodium";
#endif
